# encoding: utf-8
# Copyright (C) 2003  CAMP
# Please see the accompanying LICENSE file for further information.

"""Main gpaw module."""

import sys
from sysconfig import get_platform
from os.path import join, isfile

build_path = join(__path__[0], '..', 'build')
arch = '%s-%s' % (get_platform(), sys.version[0:3])

# If we are running the code from the source directory, then we will
# want to use the extension from the distutils build directory:
sys.path.insert(0, join(build_path, 'lib.' + arch))


from gpaw.broadcast_imports import broadcast_imports

with broadcast_imports:
    import os
    import runpy
    import warnings
    from argparse import ArgumentParser, REMAINDER, RawDescriptionHelpFormatter

    import numpy as np
    from ase.cli.run import str2dict

assert not np.version.version.startswith('1.6.0')

__version__ = '19.8.1'
__ase_version_required__ = '3.18.0'

__all__ = ['GPAW',
           'Mixer', 'MixerSum', 'MixerDif', 'MixerSum2',
           'CG', 'Davidson', 'RMMDIIS', 'DirectLCAO',
           'PoissonSolver',
           'FermiDirac', 'MethfesselPaxton',
           'PW', 'LCAO', 'restart', 'FD']


class ConvergenceError(Exception):
    pass


class KohnShamConvergenceError(ConvergenceError):
    pass


class PoissonConvergenceError(ConvergenceError):
    pass


class KPointError(Exception):
    pass


def parse_extra_parameters(arg):
    return {key.replace('-', '_'): value
            for key, value in str2dict(arg).items()}


is_gpaw_python = '_gpaw' in sys.builtin_module_names


def parse_arguments(argv):
    p = ArgumentParser(usage='%(prog)s [OPTION ...] [-c | -m] SCRIPT'
                       ' [ARG ...]',
                       description='Run a parallel GPAW calculation.\n\n'
                       'Compiled with:\n  Python {}'
                       .format(sys.version.replace('\n', '')),
                       formatter_class=RawDescriptionHelpFormatter)

    p.add_argument('--command', '-c', action='store_true',
                   help='execute Python string given as SCRIPT')
    p.add_argument('--module', '-m', action='store_true',
                   help='run library module given as SCRIPT')
    p.add_argument('-W', metavar='argument',
                   action='append', default=[], dest='warnings',
                   help='warning control.  See the documentation of -W for '
                   'the Python interpreter')
    p.add_argument('--memory-estimate-depth', default=2, type=int, metavar='N',
                   dest='memory_estimate_depth',
                   help='print memory estimate of object tree to N levels')
    p.add_argument('--domain-decomposition',
                   metavar='N or X,Y,Z', dest='parsize_domain',
                   help='use N or X × Y × Z cores for domain decomposition.')
    p.add_argument('--state-parallelization', metavar='N', type=int,
                   dest='parsize_bands',
                   help='use N cores for state/band/orbital parallelization')
    p.add_argument('--augment-grids', action='store_true',
                   dest='augment_grids',
                   help='when possible, redistribute real-space arrays on '
                   'cores otherwise used for k-point/band parallelization')
    p.add_argument('--buffer-size', type=float, metavar='SIZE',
                   help='buffer size for MatrixOperator in MiB')
    p.add_argument('--profile', metavar='FILE', dest='profile',
                   help='run profiler and save stats to FILE')
    p.add_argument('--gpaw', metavar='VAR=VALUE[, ...]', action='append',
                   default=[], dest='gpaw_extra_kwargs',
                   help='extra (hacky) GPAW keyword arguments')
    p.add_argument('--benchmark-imports', action='store_true',
                   help='count distributed/non-distributed imports')
    if is_gpaw_python:  # SCRIPT mandatory for gpaw-python
        p.add_argument('script', metavar='SCRIPT',
                       help='calculation script')
    p.add_argument('options', metavar='ARG',
                   help='arguments forwarded to SCRIPT', nargs=REMAINDER)

    args = p.parse_args(argv[1:])

    if args.command and args.module:
        p.error('-c and -m are mutually exclusive')

    if is_gpaw_python:
        sys.argv = [args.script] + args.options

    for w in args.warnings:
        # Need to convert between python -W syntax to call
        # warnings.filterwarnings():
        warn_args = w.split(':')
        assert len(warn_args) <= 5

        if warn_args[0] == 'all':
            warn_args[0] = 'always'
        if len(warn_args) >= 3:
            # e.g. 'UserWarning' (string) -> UserWarning (class)
            warn_args[2] = globals().get(warn_args[2])
        if len(warn_args) == 5:
            warn_args[4] = int(warn_args[4])

        warnings.filterwarnings(*warn_args, append=True)

    if args.parsize_domain:
        parsize = [int(n) for n in args.parsize_domain.split(',')]
        if len(parsize) == 1:
            parsize = parsize[0]
        else:
            assert len(parsize) == 3
        args.parsize_domain = parsize

    extra_parameters = {}
    for extra_kwarg in args.gpaw_extra_kwargs:
        extra_parameters.update(parse_extra_parameters(extra_kwarg))

    return extra_parameters, args


if is_gpaw_python:
    extra_parameters, gpaw_args = parse_arguments(sys.argv)
    # The normal Python interpreter puts . in sys.path, so we also do that:
    sys.path.insert(0, '.')
else:
    # Ignore the arguments; rely on --gpaw only as below.
    extra_parameters, gpaw_args = parse_arguments([sys.argv[0]])


def parse_gpaw_args():
    extra_parameters = {}
    i = 1
    while i < len(sys.argv):
        arg = sys.argv[i]
        if arg.startswith('--gpaw='):
            sys.argv.pop(i)
            extra_parameters.update(parse_extra_parameters(arg[7:]))
            continue
            break
        elif arg == '--gpaw':
            sys.argv.pop(i)
            extra_parameters.update(parse_extra_parameters(sys.argv.pop(i)))
            continue
            break
        i += 1
    return extra_parameters


extra_parameters.update(parse_gpaw_args())


# Check for special command line arguments:
memory_estimate_depth = gpaw_args.memory_estimate_depth
parsize_domain = gpaw_args.parsize_domain
parsize_bands = gpaw_args.parsize_bands
augment_grids = gpaw_args.augment_grids
# We deprecate the sl_xxx parameters being set from command line.
# People can satisfy their lusts by setting gpaw.sl_default = something
# if they are perverted enough to use global variables.
sl_default = None
sl_diagonalize = None
sl_inverse_cholesky = None
sl_lcao = None
sl_lrtddft = None
buffer_size = gpaw_args.buffer_size
profile = gpaw_args.profile


def main():
    # Stacktraces can be shortened by running script with
    # PyExec_AnyFile and friends.  Might be nicer
    if gpaw_args.command:
        d = {'__name__': '__main__'}
        exec(gpaw_args.script, d, d)
    elif gpaw_args.module:
        # Python has: python [-m MOD] [-c CMD] [SCRIPT]
        # We use a much better way: gpaw-python [-m | -c] SCRIPT
        runpy.run_module(gpaw_args.script, run_name='__main__')
    else:
        runpy.run_path(gpaw_args.script, run_name='__main__')

    # Todo, if we want: interactive interpreter.


dry_run = extra_parameters.pop('dry_run', 0)
debug = extra_parameters.pop('debug', False)

# Check for typos:
for p in extra_parameters:
    # We should get rid of most of these!
    if p not in {'sic', 'log2ng', 'PK', 'vdw0', 'df_dry_run', 'usenewlfc'}:
        warnings.warn('Unknown parameter: ' + p)

if debug:
    np.seterr(over='raise', divide='raise', invalid='raise', under='ignore')

    oldempty = np.empty

    def empty(*args, **kwargs):
        a = oldempty(*args, **kwargs)
        try:
            a.fill(np.nan)
        except ValueError:
            a.fill(-1000000)
        return a
    np.empty = empty


def get_gpaw_python_path():
    paths = os.environ['PATH'].split(os.pathsep)
    paths.insert(0, join(build_path, 'bin.' + arch))
    for path in paths:
        if isfile(join(path, 'gpaw-python')):
            return path
    raise RuntimeError('Could not find gpaw-python!')


setup_paths = []


with broadcast_imports:
    from gpaw.calculator import GPAW
    from gpaw.mixer import Mixer, MixerSum, MixerDif, MixerSum2
    from gpaw.eigensolvers import Davidson, RMMDIIS, CG, DirectLCAO
    from gpaw.poisson import PoissonSolver
    from gpaw.occupations import FermiDirac, MethfesselPaxton
    from gpaw.wavefunctions.lcao import LCAO
    from gpaw.wavefunctions.pw import PW
    from gpaw.wavefunctions.fd import FD

RMM_DIIS = RMMDIIS


def restart(filename, Class=GPAW, **kwargs):
    calc = Class(filename, **kwargs)
    atoms = calc.get_atoms()
    return atoms, calc


if profile:
    from cProfile import Profile
    import atexit
    prof = Profile()

    def f(prof, filename):
        prof.disable()
        if filename == '-':
            prof.print_stats('time')
        else:
            from gpaw.mpi import rank
            prof.dump_stats(filename + '.%04d' % rank)
    atexit.register(f, prof, profile)
    prof.enable()


command = os.environ.get('GPAWSTARTUP')
if command is not None:
    exec(command)


def is_parallel_environment():
    """Check if we are running in a parallel environment.

    This function can be redefined in ~/.gpaw/rc.py.  Example::

        def is_parallel_environment():
            import os
            return 'PBS_NODEFILE' in os.environ
    """
    return False


def read_rc_file():
    home = os.environ.get('HOME')
    if home is not None:
        rc = os.path.join(home, '.gpaw', 'rc.py')
        if os.path.isfile(rc):
            # Read file in ~/.gpaw/rc.py
            with open(rc) as fd:
                exec(fd.read())


def initialize_data_paths():
    try:
        setup_paths[:0] = os.environ['GPAW_SETUP_PATH'].split(os.pathsep)
    except KeyError:
        if len(setup_paths) == 0:
            if os.pathsep == ';':
                setup_paths[:] = [r'C:\gpaw-setups']
            else:
                setup_paths[:] = ['/usr/local/share/gpaw-setups',
                                  '/usr/share/gpaw-setups']


read_rc_file()
initialize_data_paths()



from __future__ import print_function
from ase import Atoms
from ase.units import Bohr
from gpaw import GPAW
from gpaw.test import equal

def xc(name):
    return {'name': name, 'stencil': 1}

a = 7.5 * Bohr
n = 16
atoms = Atoms('He', [(0.0, 0.0, 0.0)], cell=(a, a, a), pbc=True)
calc = GPAW(gpts=(n, n, n), nbands=1, xc=xc('PBE'))
atoms.set_calculator(calc)
e1 = atoms.get_potential_energy()
niter1 = calc.get_number_of_iterations()
e1ref = calc.get_reference_energy()
de12 = calc.get_xc_difference(xc('revPBE'))
calc.set(xc=xc('revPBE'))
e2 = atoms.get_potential_energy()
niter2 = calc.get_number_of_iterations()
e2ref = calc.get_reference_energy()
de21 = calc.get_xc_difference(xc('PBE'))
print(e1ref + e1 + de12 - (e2ref + e2))
print(e1ref + e1 - (e2ref + e2 + de21))
print(de12, de21)
equal(e1ref + e1 + de12, e2ref + e2, 8e-4)
equal(e1ref + e1, e2ref + e2 + de21, 3e-3)

calc.write('revPBE.gpw')

de21b = GPAW('revPBE.gpw').get_xc_difference(xc('PBE'))
equal(de21, de21b, 9e-8)

energy_tolerance = 0.0005
niter_tolerance = 0
equal(e1, -0.07904951, energy_tolerance)
equal(e2, -0.08147563, energy_tolerance)



from __future__ import print_function
from gpaw.fd_operators import Gradient
import numpy as np
from gpaw.grid_descriptor import GridDescriptor
from gpaw.mpi import world

if world.size > 4:
    # Grid is so small that domain decomposition cannot exceed 4 domains
    assert world.size % 4 == 0
    group, other = divmod(world.rank, 4)
    ranks = np.arange(4 * group, 4 * (group + 1))
    domain_comm = world.new_communicator(ranks)
else:
    domain_comm = world

gd = GridDescriptor((8, 1, 1), (8.0, 1.0, 1.0), comm=domain_comm)
a = gd.zeros()
dadx = gd.zeros()
a[:, 0, 0] = np.arange(gd.beg_c[0], gd.end_c[0])
gradx = Gradient(gd, v=0)
print(a.itemsize, a.dtype, a.shape)
print(dadx.itemsize, dadx.dtype, dadx.shape)
gradx.apply(a, dadx)

#   a = [ 0.  1.  2.  3.  4.  5.  6.  7.]
#
#   da
#   -- = [-2.5  1.   1.   1.   1.   1.  1.  -2.5]
#   dx

dadx = gd.collect(dadx, broadcast=True)
assert dadx[3, 0, 0] == 1.0 and np.sum(dadx[:, 0, 0]) == 0.0

gd = GridDescriptor((1, 8, 1), (1.0, 8.0, 1.0), (1, 0, 1), comm=domain_comm)
dady = gd.zeros()
a = gd.zeros()
grady = Gradient(gd, v=1)
a[0, :, 0] = np.arange(gd.beg_c[1], gd.end_c[1]) - 1
grady.apply(a, dady)

#   da
#   -- = [0.5  1.   1.   1.   1.   1.  -2.5]
#   dy

dady = gd.collect(dady, broadcast=True)
assert dady[0, 0, 0] == 0.5 and np.sum(dady[0, :, 0]) == 3.0

# a GUC cell
gd = GridDescriptor((1, 7, 1),
                    ((1.0, 0.0, 0.0),
                     (5.0, 5.0, 0.0),
                     (0.0, 0.0, 0.7)), comm=domain_comm)
dady = gd.zeros()
grady = Gradient(gd, v=1)
a = gd.zeros()
a[0, :, 0] = np.arange(gd.beg_c[1], gd.end_c[1]) - 1
grady.apply(a, dady)

#   da
#   -- = [-3.5  1.4   1.4   1.4   1.4   1.4  -3.5]
#   dy

dady = gd.collect(dady, broadcast=True)
assert abs(dady[0, 0, 0] - -3.5) < 1e-12 and abs(np.sum(dady[0, :, 0])) < 1e-12

# Check continuity of weights:
weights = []
for x in np.linspace(-0.6, 0.6, 130, True):
    gd = GridDescriptor((3, 3, 3),
                        ((3.0, 0.0, 0.0),
                         (3 * x, 1.5 * 3**0.5, 0.0),
                         (0.0, 0.0, 3)),
                        comm=domain_comm)
    g = Gradient(gd, v=0)
    for c, o in zip(g.coef_p, g.offset_pc):
        if (o == (-1, 1, 0)).all():
            break
    else:
        c = 0.0
    weights.append(c)

if 0:
    import matplotlib.pyplot as plt
    plt.plot(weights)
    plt.show()



from __future__ import print_function
import os
import tempfile
import warnings

import gpaw.mpi as mpi
from gpaw.cli.info import info
from gpaw import debug


class CLICommand:
    """Run the GPAW test suite.

    The test suite can be run in
    parallel with MPI through gpaw-python.  The test suite
    supports 1, 2, 4 or 8 CPUs although some tests are
    skipped for some parallelizations.  If no TESTs are
    given, run all tests supporting the parallelization.
    """

    @staticmethod
    def add_arguments(parser):
        add = parser.add_argument
        add('tests', nargs='*')
        add('-x', '--exclude',
            type=str, default=None,
            help='Exclude tests (comma separated list of tests).',
            metavar='test1.py,test2.py,...')
        add('-f', '--run-failed-tests-only',
            action='store_true',
            help='Run failed tests only.')
        add('--from', metavar='TESTFILE', dest='from_test',
            help='Run remaining tests, starting from TESTFILE')
        add('--after', metavar='TESTFILE', dest='after_test',
            help='Run remaining tests, starting after TESTFILE')
        add('--range',
            type=str, default=None,
            help='Run tests in range test_i.py to test_j.py '
            '(inclusive)',
            metavar='test_i.py,test_j.py')
        add('-j', '--jobs', type=int, default=1,
            help='Run JOBS threads.  Each test will be executed '
            'in serial by one thread.  This option cannot be used '
            'for parallelization together with MPI.')
        add('--reverse', action='store_true',
            help='Run tests in reverse order (less overhead with '
            'multiple jobs)')
        add('-k', '--keep-temp-dir', action='store_true',
            dest='keep_tmpdir',
            help='Do not delete temporary files.')
        add('-d', '--directory', help='Run test in this directory')
        add('-s', '--show-output', action='store_true',
            help='Show standard output from tests.')
        add('--list', action='store_true',
            help='list the full list of tests, then exit')

    @staticmethod
    def run(args):
        main(args)


def main(args):
    if len(args.tests) == 0:
        from gpaw.test import tests
    else:
        tests = args.tests

    if args.list:
        mydir, _ = os.path.split(__file__)
        for test in tests:
            print(os.path.join(mydir, test))
        return

    if args.reverse:
        tests.reverse()

    if args.run_failed_tests_only:
        tests = [line.strip() for line in open('failed-tests.txt')]

    exclude = []
    if args.exclude is not None:
        exclude += args.exclude.split(',')

    if args.from_test:
        fromindex = tests.index(args.from_test)
        tests = tests[fromindex:]

    if args.after_test:
        index = tests.index(args.after_test) + 1
        tests = tests[index:]

    if args.range:
        # default start(stop) index is first(last) test
        indices = args.range.split(',')
        try:
            start_index = tests.index(indices[0])
        except ValueError:
            start_index = 0
        try:
            stop_index = tests.index(indices[1]) + 1
        except ValueError:
            stop_index = len(tests)
        tests = tests[start_index:stop_index]

    if args.jobs > 1:
        exclude.append('maxrss.py')

    for test in exclude:
        if test in tests:
            tests.remove(test)

    from gpaw.test import TestRunner

    if mpi.world.size > 8:
        if mpi.rank == 0:
            message = (
                '!!!!!!!\n'
                'GPAW regression test suite was not designed to run on more\n'
                'than 8 MPI tasks. Re-run test suite using 1, 2, 4 or 8 MPI\n'
                'tasks instead.')
            warnings.warn(message, RuntimeWarning)

    if mpi.rank == 0:
        if args.directory is None:
            tmpdir = tempfile.mkdtemp(prefix='gpaw-test-')
        else:
            tmpdir = args.directory
            if os.path.isdir(tmpdir):
                args.keep_tmpdir = True
            else:
                os.mkdir(tmpdir)
    else:
        tmpdir = None
    tmpdir = mpi.broadcast_string(tmpdir)
    cwd = os.getcwd()
    os.chdir(tmpdir)
    if mpi.rank == 0:
        info()
        print('Running tests in', tmpdir)
        print('Jobs: {}, Cores: {}, debug-mode: {}'
              .format(args.jobs, mpi.size, debug))
    failed = TestRunner(tests, jobs=args.jobs,
                        show_output=args.show_output).run()
    os.chdir(cwd)
    mpi.world.barrier()  # syncronize before removing tmpdir
    if mpi.rank == 0:
        if len(failed) > 0:
            open('failed-tests.txt', 'w').write('\n'.join(failed) + '\n')
        if not args.keep_tmpdir:
            os.system('rm -rf ' + tmpdir)
    return failed



from __future__ import print_function

import time
import numpy as np
from gpaw.utilities.gauss import gaussian_wave

sigma = 2.4
C = 3
G = 50

t = time.time()
r_cG = np.random.normal(size=C*G**3).reshape((C,G,G,G))
r0_c = np.random.normal(size=C)
k_c = np.random.normal(size=C)
A = np.random.uniform()*np.exp(1j*np.random.uniform(0,2*np.pi))
print('Allocation: %8.5f s' % (time.time()-t))

# -------------------------------------------------------------------

# Test case for real-part of gamma-point wave with normalized amplitude
_gaussRGN = lambda r_cG, r0_c, sigma: 1/(sigma*np.pi**0.5)**1.5 \
    * np.exp(-np.sum((r_cG-r0_c[:,np.newaxis,np.newaxis,np.newaxis])**2, axis=0)/(2*sigma**2))

t = time.time()
gs0_G = _gaussRGN(r_cG, r0_c, sigma)
print('_gaussRGN: %8.5f s' % (time.time()-t))

t = time.time()
gs1_G = gaussian_wave(r_cG, r0_c, sigma)
print('+gaussRGN: %8.5f s' % (time.time()-t))

assert np.abs(gs0_G-gs1_G).max() < 1e-12, 'Max error %g' % np.abs(gs0_G-gs1_G).max()
del gs0_G, gs1_G

# Test case for real-part of gamma-point wave with complex amplitude
_gaussRGA = lambda r_cG, r0_c, sigma, A: np.real(A) \
    * np.exp(-np.sum((r_cG-r0_c[:,np.newaxis,np.newaxis,np.newaxis])**2, axis=0)/(2*sigma**2))

t = time.time()
gs0_G = _gaussRGA(r_cG, r0_c, sigma, A)
print('_gaussRGA: %8.5f s' % (time.time()-t))

t = time.time()
gs1_G = gaussian_wave(r_cG, r0_c, sigma, None, A)
print('+gaussRGA: %8.5f s' % (time.time()-t))

assert np.abs(gs0_G-gs1_G).max() < 1e-12, 'Max error %g' % np.abs(gs0_G-gs1_G).max()
del gs0_G, gs1_G

# Test case for real-part of kpoint-point wave with normalized amplitude
_gaussRKN = lambda r_cG, r0_c, sigma, k_c: 1/(sigma*np.pi**0.5)**1.5 \
    * np.exp(-np.sum((r_cG-r0_c[:,np.newaxis,np.newaxis,np.newaxis])**2, axis=0)/(2*sigma**2)) \
    * np.cos(np.sum(r_cG*k_c[:,np.newaxis,np.newaxis,np.newaxis], axis=0))

t = time.time()
gs0_G = _gaussRKN(r_cG, r0_c, sigma, k_c)
print('_gaussRKN: %8.5f s' % (time.time()-t))

t = time.time()
gs1_G = gaussian_wave(r_cG, r0_c, sigma, k_c)
print('+gaussRKN: %8.5f s' % (time.time()-t))

assert np.abs(gs0_G-gs1_G).max() < 1e-12, 'Max error %g' % np.abs(gs0_G-gs1_G).max()
del gs0_G, gs1_G

# Test case for real-part of kpoint-point wave with complex amplitude
_gaussRKA = lambda r_cG, r0_c, sigma, k_c, A: \
    np.exp(-np.sum((r_cG-r0_c[:,np.newaxis,np.newaxis,np.newaxis])**2, axis=0)/(2*sigma**2)) \
    * np.real(A*np.exp(1j*np.sum(r_cG*k_c[:,np.newaxis,np.newaxis,np.newaxis], axis=0)))

t = time.time()
gs0_G = _gaussRKA(r_cG, r0_c, sigma, k_c, A)
print('_gaussRKA: %8.5f s' % (time.time()-t))

t = time.time()
gs1_G = gaussian_wave(r_cG, r0_c, sigma, k_c, A)
print('+gaussRKA: %8.5f s' % (time.time()-t))

assert np.abs(gs0_G-gs1_G).max() < 1e-12, 'Max error %g' % np.abs(gs0_G-gs1_G).max()
del gs0_G, gs1_G

# -------------------------------------------------------------------

# Test case for complex case of gamma-point wave with normalized amplitude
_gaussCGN = lambda r_cG, r0_c, sigma: (1+0j)/(sigma*np.pi**0.5)**1.5 \
    * np.exp(-np.sum((r_cG-r0_c[:,np.newaxis,np.newaxis,np.newaxis])**2, axis=0)/(2*sigma**2))

t = time.time()
gs0_G = _gaussCGN(r_cG, r0_c, sigma)
print('_gaussCGN: %8.5f s' % (time.time()-t))

t = time.time()
gs1_G = gaussian_wave(r_cG, r0_c, sigma, dtype=complex)
print('+gaussCGN: %8.5f s' % (time.time()-t))

assert np.abs(gs0_G-gs1_G).max() < 1e-12, 'Max error %g' % np.abs(gs0_G-gs1_G).max()
del gs0_G, gs1_G

# Test case for complex case of gamma-point wave with complex amplitude
_gaussCGA = lambda r_cG, r0_c, sigma, A: A \
    * np.exp(-np.sum((r_cG-r0_c[:,np.newaxis,np.newaxis,np.newaxis])**2, axis=0)/(2*sigma**2))

t = time.time()
gs0_G = _gaussCGA(r_cG, r0_c, sigma, A)
print('_gaussCGA: %8.5f s' % (time.time()-t))

t = time.time()
gs1_G = gaussian_wave(r_cG, r0_c, sigma, None, A, dtype=complex)
print('+gaussCGA: %8.5f s' % (time.time()-t))

assert np.abs(gs0_G-gs1_G).max() < 1e-12, 'Max error %g' % np.abs(gs0_G-gs1_G).max()
del gs0_G, gs1_G

# Test case for complex case of kpoint-point wave with normalized amplitude
_gaussCKN = lambda r_cG, r0_c, sigma, k_c: (1+0j)/(sigma*np.pi**0.5)**1.5 \
    * np.exp(-np.sum((r_cG-r0_c[:,np.newaxis,np.newaxis,np.newaxis])**2, axis=0)/(2*sigma**2)) \
    * np.exp(1j*np.sum(r_cG*k_c[:,np.newaxis,np.newaxis,np.newaxis], axis=0))

t = time.time()
gs0_G = _gaussCKN(r_cG, r0_c, sigma, k_c)
print('_gaussCKN: %8.5f s' % (time.time()-t))

t = time.time()
gs1_G = gaussian_wave(r_cG, r0_c, sigma, k_c, dtype=complex)
print('+gaussCKN: %8.5f s' % (time.time()-t))

assert np.abs(gs0_G-gs1_G).max() < 1e-12, 'Max error %g' % np.abs(gs0_G-gs1_G).max()
del gs0_G, gs1_G

# Test case for complex case of kpoint-point wave with complex amplitude
_gaussCKA = lambda r_cG, r0_c, sigma, k_c, A: A \
    * np.exp(-np.sum((r_cG-r0_c[:,np.newaxis,np.newaxis,np.newaxis])**2, axis=0)/(2*sigma**2)) \
    * np.exp(1j*np.sum(r_cG*k_c[:,np.newaxis,np.newaxis,np.newaxis], axis=0))

t = time.time()
gs0_G = _gaussCKA(r_cG, r0_c, sigma, k_c, A)
print('_gaussCKA: %8.5f s' % (time.time()-t))

t = time.time()
gs1_G = gaussian_wave(r_cG, r0_c, sigma, k_c, A, dtype=complex)
print('+gaussCKA: %8.5f s' % (time.time()-t))

assert np.abs(gs0_G-gs1_G).max() < 1e-12, 'Max error %g' % np.abs(gs0_G-gs1_G).max()
del gs0_G, gs1_G




from gpaw import GPAW, restart
from ase.build import bcc111
from gpaw.poisson import PoissonSolver
import numpy as np

slab = bcc111('Na', (1, 1, 2), vacuum=8)
electrodes = ['both', 'single']
charge = 0.05

for electrode in electrodes:
    slab.calc = GPAW(xc='LDA', h=0.22,
                        txt= 'metallic.txt', charge = charge,
                        convergence = {'density': 1e-1, 'energy': 1e-1, 'eigenstates': 1e-1},
                        kpts=(2, 2, 1),
                        poissonsolver=PoissonSolver(metallic_electrodes=electrode))

    E = slab.get_potential_energy()
    phi0 = slab.calc.get_electrostatic_potential()
    if electrode == 'single':
        assert np.all(abs(phi0[:,:,0])<1e-10)
    else:
        print(phi0[:,:,0])
        print(phi0[:,:,1])
        assert np.all(abs(phi0[:,:,0])<1e-10)
        # The last zero boundary condition is implicit, so extrapolate
        d = phi0[:,:,-1]-phi0[:,:,-2]
        assert np.all(abs(phi0[:,:,-1]+d)<1e-5)

    slab.calc.write('%s.gpw' % electrode)

    atoms, calc = restart('%s.gpw' % electrode, txt='restart.txt')
    phi02 = calc.get_electrostatic_potential()
    assert np.all(abs(phi02-phi0)<1e-10)



import numpy as np
from gpaw import GPAW, PW, Mixer, Davidson
from ase.parallel import parprint
from ase.lattice.compounds import L1_2

name = 'Cu3Au'
structure = 'L1_2'
ecut = 300
kpts = (2, 2, 2)

QNA = {'alpha': 2.0,
       'name': 'QNA',
       'stencil': 1,
       'orbital_dependent': False,
       'parameters': {'Au': (0.125, 0.1), 'Cu': (0.0795, 0.005)},
       'setup_name': 'PBE',
       'type': 'qna-gga'}

atoms = L1_2(['Au', 'Cu'], latticeconstant=3.7)

calc = GPAW(mode=PW(ecut),
            eigensolver=Davidson(2),
            nbands='120%',
            mixer=Mixer(0.4, 7, 50.0),
            parallel=dict(domain=1),
            xc=QNA,
            kpts=kpts,
            txt=name + '.txt')

atoms.set_calculator(calc)

atoms.set_cell(np.dot(atoms.cell,
                      [[1.02, 0, 0.03],
                       [0, 0.99, -0.02],
                       [0.2, -0.01, 1.03]]),
               scale_atoms=True)

s_analytical = atoms.get_stress()
s_numerical = atoms.calc.calculate_numerical_stress(atoms, 1e-5)
s_err = s_numerical - s_analytical

parprint('Analytical stress:', s_analytical)
parprint('Numerical stress :', s_numerical)
parprint('Error in stress  :', s_err)
assert np.all(abs(s_err) < 5e-4)



from ase.build import molecule
from gpaw import GPAW, PW
from gpaw.mpi import world

a = molecule('H', pbc=1)
a.center(vacuum=2)

comm = world.new_communicator([world.rank])
e0 = 0.0
a.calc = GPAW(mode=PW(250),
              communicator=comm,
              txt=None)
e0 = a.get_potential_energy()
e0 = world.sum(e0) / world.size

a.calc = GPAW(mode=PW(250),
              eigensolver='rmm-diis',
              basis='szp(dzp)',
              txt='%d.txt' % world.size)
e = a.get_potential_energy()
f = a.get_forces()
assert abs(e - e0) < 7e-5, abs(e - e0)
assert abs(f).max() < 1e-10, abs(f).max()



from __future__ import division
import numpy as np
from ase import Atoms
from ase.calculators.test import numeric_force
from gpaw import GPAW, PoissonSolver, Mixer
from gpaw.external import PointChargePotential
from gpaw.test import equal
import _gpaw

# Find coefs for polynomial:
c = np.linalg.solve([[1, 1, 1, 1],
                     [0, 2, 4, 6],
                     [0, 2, 12, 30],
                     [1 / 3, 1 / 5, 1 / 7, 1 / 9]],
                    [1, -1, 2, 0.5])
print(c)
print(c * 32)
x = np.linspace(0, 1, 101)
v = np.polyval(c[::-1], x**2)
equal((v * x**2 - x).sum() / 100, 0, 1e-5)

if 0:
    import matplotlib.pyplot as plt
    plt.plot(x, v)
    x = np.linspace(0.2, 1.5, 101)
    plt.plot(x, 1 / x)
    plt.show()

# Low-level test:
h = 0.1
q = 2.2
beg_v = np.zeros(3, int)
h_v = np.ones(3) * h
q_p = np.ones(1) * q
R_pv = np.array([[0.1, 0.2, -0.3]])
vext_G = np.zeros((1, 1, 1))
rhot_G = np.ones((1, 1, 1))


def f(rc):
    vext_G[:] = 0.0
    _gpaw.pc_potential(beg_v, h_v, q_p, R_pv, rc, np.inf, 1.0, vext_G, None)
    return -vext_G[0, 0, 0]

d = (R_pv[0]**2).sum()**0.5

for rc in [0.9 * d, 1.1 * d]:
    if d > rc:
        v0 = q / d
    else:
        v0 = np.polyval(c[::-1], (d / rc)**2) * q / rc

    equal(f(rc), v0, 1e-12)

    F_pv = np.zeros((1, 3))
    _gpaw.pc_potential(beg_v, h_v, q_p, R_pv, rc, np.inf, 1.0,
                       vext_G, None, rhot_G, F_pv)
    eps = 0.0001
    for v in range(3):
        R_pv[0, v] += eps
        ep = f(rc)
        R_pv[0, v] -= 2 * eps
        em = f(rc)
        R_pv[0, v] += eps
        F = -(ep - em) / (2 * eps) * h**3
        equal(F, -F_pv[0, v], 1e-9)

# High-level test:
lih = Atoms('LiH')
lih[1].y += 1.64
lih.center(vacuum=3)

pos = lih.cell.sum(axis=0)
print(pos)
pc = PointChargePotential([-1.0], [pos])
lih.calc = GPAW(external=pc,
                mixer=Mixer(0.8, 5, 20.0),
                xc='oldLDA',
                poissonsolver=PoissonSolver(),
                convergence=dict(density=3e-6),
                txt='lih-pc.txt')
f1 = lih.get_forces()
fpc1 = pc.get_forces(lih.calc)
print(fpc1)
print(fpc1 + f1.sum(0))

f2 = [[numeric_force(lih, a, v) for v in range(3)] for a in range(2)]
print(f1)
print(f1 - f2)
err = abs(f1 - f2).max()
print('err', err)
assert err < 2e-3, err

x = 0.0001
for v in range(3):
    pos[v] += x
    pc.set_positions([pos])
    ep = lih.get_potential_energy()
    pos[v] -= 2 * x
    pc.set_positions([pos])
    em = lih.get_potential_energy()
    pos[v] += x
    pc.set_positions([pos])
    error = (em - ep) / (2 * x) - fpc1[0, v]
    print(v, error)
    assert abs(error) < 0.006



"""Calculate diamond with various parallelizations with GLLBSC."""
from __future__ import print_function
from ase.build import bulk
from ase.units import Ha
from gpaw import GPAW, restart, Mixer
from gpaw.test import gen
from gpaw import setup_paths


xc = 'GLLBSC'
gen('C', xcname=xc)
setup_paths.insert(0, '.')

KSb = []
dxcb = []

eigensolver = 'rmm-diis'

for band in [1, 2, 4]:
    # Calculate ground state
    atoms = bulk('C', 'diamond', a=3.567)
    calc = GPAW(h=0.15,
                kpts=(4, 4, 4),
                xc=xc,
                nbands=8,
                mixer=Mixer(0.5, 5, 50.0),
                eigensolver=eigensolver,
                parallel={'band': band})
    atoms.set_calculator(calc)
    atoms.get_potential_energy()
    calc.write('Cgs.gpw')

    # Calculate accurate KS-band gap from band structure
    calc = GPAW('Cgs.gpw',
                kpts={'path': 'GX', 'npoints': 12},
                fixdensity=True,
                symmetry='off',
                nbands=8,
                convergence=dict(bands=8),
                eigensolver=eigensolver,
                parallel={'band': band})
    calc.get_atoms().get_potential_energy()
    # Get the accurate KS-band gap
    homolumo = calc.get_homo_lumo()
    homo, lumo = homolumo
    print('band gap', lumo - homo)

    # Redo the ground state calculation
    calc = GPAW(h=0.15,
                kpts=(4, 4, 4),
                xc=xc,
                nbands=8,
                mixer=Mixer(0.5, 5, 50.0),
                eigensolver=eigensolver,
                parallel={'band': band})
    atoms.set_calculator(calc)
    atoms.get_potential_energy()
    # And calculate the discontinuity potential with accurate band gap
    response = calc.hamiltonian.xc.xcs['RESPONSE']
    response.calculate_delta_xc(homolumo=homolumo / Ha)
    calc.write('CGLLBSC.gpw')

    # Redo the band structure calculation
    atoms, calc = restart('CGLLBSC.gpw',
                          kpts={'path': 'GX', 'npoints': 12},
                          fixdensity=True,
                          symmetry='off',
                          convergence=dict(bands=8))
    atoms.get_potential_energy()
    response = calc.hamiltonian.xc.xcs['RESPONSE']
    KS, dxc = response.calculate_delta_xc_perturbation()
    KSb.append(KS)
    dxcb.append(dxc)
    assert abs(KS + dxc - 5.41) < 0.10
    # M. Kuisma et. al, Phys. Rev. B 82, 115106,
    # QP gap for C, 5.41eV, expt. 5.48eV

assert abs(KSb[0] - KSb[1]) < 1e-6
assert abs(KSb[0] - KSb[2]) < 1e-6
assert abs(dxcb[0] - dxcb[1]) < 1e-6
assert abs(dxcb[0] - dxcb[2]) < 1e-6



from __future__ import print_function

from ase import Atoms, Atom
from ase.parallel import barrier, rank
from gpaw.cluster import Cluster
from gpaw.test import equal
from ase.build import molecule
from math import sqrt

R = 2.0
CO = Atoms([Atom('C', (1, 0, 0)), Atom('O', (1, 0, R))])

CO.rotate(90, 'y')
equal(CO.positions[1, 0], R, 1e-10)

# translate
CO.translate(-CO.get_center_of_mass())
p = CO.positions.copy()
for i in range(2):
    equal(p[i, 1], 0, 1e-10)
    equal(p[i, 2], 0, 1e-10)

# rotate the nuclear axis to the direction (1,1,1)
CO.rotate(p[1] - p[0], (1, 1, 1))
q = CO.positions.copy()
for c in range(3):
    equal(q[0, c], p[0, 0] / sqrt(3), 1e-10)
    equal(q[1, c], p[1, 0] / sqrt(3), 1e-10)

# minimal box
b = 4.0
CO = Cluster([Atom('C', (1, 0, 0)), Atom('O', (1, 0, R))])
CO.minimal_box(b)
cc = CO.get_cell()
for c in range(3):
    width = 2 * b
    if c == 2:
        width += R
    equal(cc[c, c], width, 1e-10)

# minimal box, ensure multiple of 4
h = .13
b = [2, 3, 4]
CO.minimal_box(b, h=h)
cc = CO.get_cell()
for c in range(3):
    # print "cc[c,c], cc[c,c] / h % 4 =", cc[c, c], cc[c, c] / h % 4
    for a in CO:
        print(a.symbol, b[c], a.position[c], cc[c, c] - a.position[c])
        assert(a.position[c] > b[c])
    equal(cc[c, c] / h % 4, 0.0, 1e-10)

# connected atoms
assert(len(CO.find_connected(0, 1.1 * R)) == 2)
assert(len(CO.find_connected(0, 0.9 * R)) == 1)

H2O = Cluster(molecule('H2O'))
assert (len(H2O.find_connected(0)) == 3)
assert (len(H2O.find_connected(0, scale=0.9)) == 1)

# I/O
fxyz = 'CO.xyz'
fpdb = 'CO.pdb'

cell = [2., 3., R + 2.]
CO.set_cell(cell, scale_atoms=True)
barrier()
CO.write(fxyz)
barrier()
CO_b = Cluster(filename=fxyz)
assert(len(CO) == len(CO_b))
offdiagonal = CO_b.get_cell().sum() - CO_b.get_cell().diagonal().sum()
assert(offdiagonal == 0.0)

barrier()
CO.write(fpdb)

# read xyz files with additional info
read_with_additional = True
if read_with_additional:
    if rank == 0:
        f = open(fxyz, 'w')
        print("""2

C 0 0 0. 1 2 3
O 0 0 1. 6. 7. 8.""", file=f)
        f.close()

    barrier()

    CO = Cluster(filename=fxyz)



from __future__ import print_function
import numpy as np

from gpaw.atom.radialgd import fsbt, RadialGridDescriptor as RGD


N = 1024
L = 50.0
h = L / N
alpha = 5.0
r = np.linspace(0, L, N, endpoint=False)
G = np.linspace(0, np.pi / h, N // 2 + 1)
n = np.exp(-alpha * r**2)

for l in range(7):
    f = fsbt(l, n * r**l, r, G)
    f0 = (np.pi**0.5 / alpha**(l + 1.5) / 2**l * G**l / 4 *
          np.exp(-G**2 / (4 * alpha)))
    tol = 3e-6 * 10**(-7 + l)
    print(l, abs(f - f0).max(), 'tol=', tol)
    assert abs(f - f0).max() < tol

rgd = RGD(r, r * 0 + r[1])
g, f = rgd.fft(n * r)
f0 = 4 * np.pi**1.5 / alpha**1.5 / 4 * np.exp(-g**2 / 4 / alpha)
assert abs(f - f0).max() < 1e-6

# This is how to do the inverse FFT:
ggd = RGD(g, g * 0 + g[1])
r, f = ggd.fft(f * g)
assert abs(np.exp(-alpha * r**2) - f / 8 / np.pi**3).max() < 2e-3



import numpy as np
from ase.build import molecule
from ase.dft import Wannier
from gpaw import GPAW
from gpaw.test import equal

# Test of ase wannier using gpaw

calc = GPAW(gpts=(32, 32, 32), nbands=4)
atoms = molecule('H2', calculator=calc)
atoms.center(vacuum=3.)
e = atoms.get_potential_energy()
niter = calc.get_number_of_iterations()

pos = atoms.positions + np.array([[0, 0, .2339], [0, 0, -.2339]])
com = atoms.get_center_of_mass()

wan = Wannier(nwannier=2, calc=calc, initialwannier='bloch')
equal(wan.get_functional_value(), 2.964, 1e-3)
equal(np.linalg.norm(wan.get_centers() - [com, com]), 0, 1e-4)

wan = Wannier(nwannier=2, calc=calc, initialwannier='projectors')
equal(wan.get_functional_value(), 3.100, 2e-3)
equal(np.linalg.norm(wan.get_centers() - pos), 0, 1e-3)

wan = Wannier(nwannier=2, calc=calc, initialwannier=[[0, 0, .5], [1, 0, .5]])
equal(wan.get_functional_value(), 3.100, 2e-3)
equal(np.linalg.norm(wan.get_centers() - pos), 0, 1e-3)

wan.localize()
equal(wan.get_functional_value(), 3.100, 2e-3)
equal(np.linalg.norm(wan.get_centers() - pos), 0, 1e-3)
equal(np.linalg.norm(wan.get_radii() - 1.2393), 0, 2e-3)
eig = np.sort(np.linalg.eigvals(wan.get_hamiltonian().real))
equal(np.linalg.norm(eig - calc.get_eigenvalues()[:2]), 0, 1e-4)

wan.write_cube(0, 'H2.cube')

energy_tolerance = 0.002
niter_tolerance = 0
equal(e, -6.652, energy_tolerance)



from __future__ import print_function
from ase import Atom, Atoms
from ase.calculators.test import numeric_force
from gpaw import GPAW, Mixer, FermiDirac, Davidson
from gpaw.test import equal

a = 4.0
n = 16
atoms = Atoms([Atom('H', [1.234, 2.345, 3.456])],
                    cell=(a, a, a), pbc=True)
calc = GPAW(nbands=1,
            gpts=(n, n, n),
            txt=None,
            eigensolver=Davidson(4),
            mixer=Mixer(0.3, 3, 1),
            convergence={'energy': 1e-7},
            occupations=FermiDirac(0.0))
atoms.set_calculator(calc)
e1 = atoms.get_potential_energy()
niter1 = calc.get_number_of_iterations()
f1 = atoms.get_forces()[0]
for i in range(3):
    f2i = numeric_force(atoms, 0, i)
    print(f1[i], f2i)
    equal(f1[i], f2i, 0.00025)

energy_tolerance = 0.001
force_tolerance = 0.004
niter_tolerance = 0
equal(e1, -0.5318, energy_tolerance)
f1_ref = [-0.29138, -0.3060, -0.3583]
for i in range(3):
    equal(f1[i], f1_ref[i], force_tolerance)



from __future__ import print_function
from math import sqrt, pi

import numpy as np
from ase import Atoms
from ase.units import Bohr, Hartree
from ase.parallel import rank, size

from gpaw import GPAW
from gpaw.external import ConstantElectricField
from gpaw.point_charges import PointCharges
from gpaw.utilities import packed_index
from gpaw.pair_density import PairDensity

# Three ways to compute the polarizability of hydrogen:
# 1. Perturbation theory
# 2. Constant electric field
# 3. Middle of an electric dipole

# Note: The analytical value for the polarizability
# is 4.5 a0**3 (e.g. PRA 33, 3671), while the experimental
# value is 4.6 a0**3 (e.g. PR 133, A629).

to_au = Hartree / Bohr**2
to_eVA = Hartree / Bohr


def dipole_op(c, state1, state2, k=0, s=0):
    # Taken from KSSingle, maybe make this accessible in
    # KSSingle?
    wfs = c.wfs
    gd = wfs.gd

    kpt = None
    for i in wfs.kpt_u:
        if i.k == k and i.s == s:
            kpt = i
    
    pd = PairDensity(c)
    pd.initialize(kpt, state1, state2)

    # coarse grid contribution
    # <i|r|j> is the negative of the dipole moment (because of negative
    # e- charge)
    me = -gd.calculate_dipole_moment(pd.get())

    # augmentation contributions
    ma = np.zeros(me.shape)
    pos_av = c.get_atoms().get_positions() / Bohr
    for a, P_ni in kpt.P_ani.items():
        Ra = pos_av[a]
        Pi_i = P_ni[state1]
        Pj_i = P_ni[state2]
        Delta_pL = wfs.setups[a].Delta_pL
        ni = len(Pi_i)

        ma0 = 0
        ma1 = np.zeros(me.shape)
        for i in range(ni):
            for j in range(ni):
                pij = Pi_i[i] * Pj_i[j]
                ij = packed_index(i, j, ni)
                # L=0 term
                ma0 += Delta_pL[ij, 0] * pij
                # L=1 terms
                if wfs.setups[a].lmax >= 1:
                    # see spherical_harmonics.py for
                    # L=1:y L=2:z; L=3:x
                    ma1 += np.array([Delta_pL[ij, 3], Delta_pL[ij, 1],
                                     Delta_pL[ij, 2]]) * pij
        ma += sqrt(4 * pi / 3) * ma1 + Ra * sqrt(4 * pi) * ma0
    gd.comm.sum(ma)

    me += ma

    return me * Bohr


# Currently only works on a single processor
assert size == 1

maxfield = 0.01
nfs = 5          # number of field
nbands = 30      # number of bands
h = 0.20         # grid spacing

debug = not False

if debug:
    txt = 'gpaw.out'
else:
    txt = None

test1 = True
test2 = True
test3 = True

a0 = 6.0
a = Atoms('H',
          positions=[[a0 / 2, a0 / 2, a0 / 2 ]],
          cell=[a0, a0, a0])

alpha1 = None
alpha2 = None
alpha3 = None

# Test 1

if test1:
    c = GPAW(
        h=h,
        nbands=nbands + 10,
        spinpol=True,
        hund=True,
        xc='LDA',
        eigensolver='cg',
        convergence={'bands': nbands, 'eigenstates': 3.3e-4},
        maxiter=1000,
        txt=txt)
    a.set_calculator(c)
    a.get_potential_energy()

    o1 = c.get_occupation_numbers(spin=0)

    if o1[0] > 0.0:
        spin = 0
    else:
        spin = 1

    alpha = 0.0

    ev = c.get_eigenvalues(0, spin)
    gd = c.wfs.gd
    for i in range(1, nbands):
        mu_x, mu_y, mu_z = dipole_op(c, 0, i, k=0, s=spin)

        alpha += mu_z**2 / (ev[i] - ev[0])

    alpha *= 2

    if rank == 0 and debug:
        print('From perturbation theory:')
        print('  alpha = ', alpha, ' A**2/eV')
        print('  alpha = ', alpha * to_au, ' Bohr**3')

    alpha1 = alpha

###

c = GPAW(
    h=h,
    nbands       = 2,
    spinpol      = True,
    hund         = True,
    xc           = 'LDA',
    #eigensolver  = 'cg',
    convergence  = {'bands': nbands, 'eigenstates': 3.3e-4},
    maxiter      = 1000,
    txt          = txt)
a.set_calculator(c)

# Test 2

if test2:
    e       = [ ]
    e1s     = [ ]
    d       = [ ]
    fields  = np.linspace(-maxfield, maxfield, nfs)
    for field in fields:
        if rank == 0 and debug:
            print(field)
        c.set(
            external  = ConstantElectricField(field),
            )
        etot = a.get_potential_energy()
        e   += [ etot ]
        ev0  = c.get_eigenvalues(0)
        ev1  = c.get_eigenvalues(1)
        e1s += [ min( ev0[0], ev1[0] ) ]
        dip  = c.get_dipole_moment()
        d   += [ dip[2] ]


    pol1, dummy = np.polyfit(fields, d, 1)
    pol2, dummy1, dummy2 = np.polyfit(fields, e1s, 2)

    if rank == 0 and debug:
        print('From shift in 1s-state at constant electric field:')
        print('  alpha = ', -pol2, ' A**2/eV')
        print('  alpha = ', -pol2*to_au, ' Bohr**3')

        print('From dipole moment at constant electric field:')
        print('  alpha = ', pol1, ' A**2/eV')
        print('  alpha = ', pol1*to_au, ' Bohr**3')

        np.savetxt('ecf.out', np.transpose( [ fields, e, e1s, d ] ))

    assert abs(pol1+pol2) < 0.0001

    alpha2 = (pol1-pol2)/2


# Test 3

if test3:
    pcd        = 1000.0    # distance of the two point charges
    maxcharge  = 100.0     # maximum charge on the point charge

    e        = [ ]
    e1s      = [ ]
    d        = [ ]
    charges  = np.linspace(-maxcharge, maxcharge, nfs)
    fields   = [ ]
    for charge in charges:
        ex = PointCharges(
            positions  = [ [ a0/2, a0/2, -pcd/2+a0/2 ], [ a0/2, a0/2, pcd/2+a0/2 ] ],
            charges    = [ charge, -charge ]
            )
        c.set(
            external  = ex
            )
        etot = a.get_potential_energy()
        e      += [ etot ]
        ev0     = c.get_eigenvalues(0)
        ev1     = c.get_eigenvalues(1)
        e1s    += [ min( ev0[0], ev1[0] ) ]
        dip     = c.get_dipole_moment()
        d      += [ dip[2] ]
        field   = ex.get_taylor(position=a[0].position)[1][1]
        if rank == 0 and debug:
            print(field*to_eVA, 2*charge/((pcd/2)**2)*Hartree*Bohr)
        fields += [ field*to_eVA ]

    pol1, dummy = np.polyfit(fields, d, 1)
    pol2, dummy1, dummy2 = np.polyfit(fields, e1s, 2)

    if rank == 0 and debug:
        print('From shift in 1s-state between two point charges:')
        print('  alpha = ', -pol2, ' A**2/eV')
        print('  alpha = ', -pol2*to_au, ' Bohr**3')

        #print 'From dipole moment between two point charges:'
        #print '  alpha = ', pol1, ' A**2/eV'
        #print '  alpha = ', pol1*to_au, ' Bohr**3'

        np.savetxt('epc.out', np.transpose( [ fields, e, e1s, d ] ))

    alpha3 = alpha

# This is a very, very rough test
assert abs(alpha1-alpha2) < 0.01
assert abs(alpha3-alpha2) < 0.01



import numpy as np

from gpaw.utilities import pack2, unpack2


def hubbard(setup, D_sp):
    nspins = len(D_sp)

    l_j = setup.l_j
    l = setup.Hubl
    scale = setup.Hubs
    nl = np.where(np.equal(l_j, l))[0]
    nn = (2 * np.array(l_j) + 1)[0:nl[0]].sum()

    e_xc = 0.0
    dH_sp = []

    s = 0
    for D_p in D_sp:
        N_mm, V = aoom(setup, unpack2(D_p), l, scale)
        N_mm = N_mm / 2 * nspins

        if nspins == 4:
            N_mm = N_mm / 2.0
            if s == 0:
                Eorb = setup.HubU / 2. * (N_mm - 
                                    0.5 * np.dot(N_mm, N_mm)).trace()

                Vorb = setup.HubU / 2. * (np.eye(2 * l +1) -N_mm)

            else:
                Eorb = setup.HubU / 2. * (-0.5*np.dot(N_mm , N_mm)).trace()

                Vorb = -setup.HubU / 2. * N_mm
        else:        
            Eorb = setup.HubU / 2. * (N_mm -
                                np.dot(N_mm, N_mm)).trace()

            Vorb = setup.HubU * (0.5 * np.eye(2 * l + 1) - N_mm)

        e_xc += Eorb
        if nspins == 1:
            # add contribution of other spin manyfold
            e_xc += Eorb

        if len(nl) == 2:
            mm = (2 * np.array(l_j) + 1)[0:nl[1]].sum()

            V[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] *= Vorb
            V[mm:mm + 2 * l + 1, nn:nn + 2 * l + 1] *= Vorb
            V[nn:nn + 2 * l + 1, mm:mm + 2 * l + 1] *= Vorb
            V[mm:mm + 2 * l + 1, mm:mm + 2 * l + 1] *= Vorb
        else:
            V[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] *= Vorb

        dH_sp.append(pack2(V))
        s += 1
        
    return e_xc, dH_sp


def aoom(setup, DM, l, scale=1):
    """Atomic Orbital Occupation Matrix.

    Determine the Atomic Orbital Occupation Matrix (aoom) for a
    given l-quantum number.

    This operation, takes the density matrix (DM), which for
    example is given by unpack2(D_asq[i][spin]), and corrects for
    the overlap between the selected orbitals (l) upon which the
    the density is expanded (ex <p|p*>,<p|p>,<p*|p*> ).

    Returned is only the "corrected" part of the density matrix,
    which represents the orbital occupation matrix for l=2 this is
    a 5x5 matrix.
    """
    S = setup
    l_j = S.l_j
    lq = S.lq
    nl = np.where(np.equal(l_j, l))[0]
    V = np.zeros(np.shape(DM))
    if len(nl) == 2:
        aa = nl[0] * len(l_j) - (nl[0] - 1) * nl[0] // 2
        bb = nl[1] * len(l_j) - (nl[1] - 1) * nl[1] // 2
        ab = aa + nl[1] - nl[0]

        if not scale:
            lq_a = lq[aa]
            lq_ab = lq[ab]
            lq_b = lq[bb]
        else:
            lq_a = 1
            lq_ab = lq[ab] / lq[aa]
            lq_b = lq[bb] / lq[aa]

        # and the correct entrances in the DM
        nn = (2 * np.array(l_j) + 1)[0:nl[0]].sum()
        mm = (2 * np.array(l_j) + 1)[0:nl[1]].sum()

        # finally correct and add the four submatrices of NC_DM
        A = DM[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] * lq_a
        B = DM[nn:nn + 2 * l + 1, mm:mm + 2 * l + 1] * lq_ab
        C = DM[mm:mm + 2 * l + 1, nn:nn + 2 * l + 1] * lq_ab
        D = DM[mm:mm + 2 * l + 1, mm:mm + 2 * l + 1] * lq_b

        V[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] = lq_a
        V[nn:nn + 2 * l + 1, mm:mm + 2 * l + 1] = lq_ab
        V[mm:mm + 2 * l + 1, nn:nn + 2 * l + 1] = lq_ab
        V[mm:mm + 2 * l + 1, mm:mm + 2 * l + 1] = lq_b

        return A + B + C + D, V
    else:
        assert len(nl) == 1
        assert l_j[-1] == l
        nn = (2 * np.array(l_j) + 1)[0:nl[0]].sum()
        A = DM[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] * lq[-1]
        V[nn:nn + 2 * l + 1, nn:nn + 2 * l + 1] = lq[-1]
        return A, V



from ase.build import bulk
from gpaw import GPAW, LCAO

# This test calculates a GLLB quasiparticle gap with LCAO and verifies
# that it does not change from a reference value.  Note that the
# calculation, physically speaking, is garbage.

si = bulk('Si', 'diamond', a=5.421)
calc = GPAW(mode=LCAO(interpolation=2),
            h=0.3,
            basis='sz(dzp)',
            xc='GLLBSC',
            kpts={'size': (2, 2, 2), 'gamma': True},
            txt='si.txt')


def stopcalc():
    calc.scf.converged = True


calc.attach(stopcalc, 1)

si.set_calculator(calc)
si.get_potential_energy()

response = calc.hamiltonian.xc.xcs['RESPONSE']
response.calculate_delta_xc()
EKs, Dxc = response.calculate_delta_xc_perturbation()
refgap = 3.02333
gap = EKs + Dxc
print('GAP', gap)
err = abs(gap - refgap)
assert err < 1e-4, err



from __future__ import division
import numpy as np
from ase import Atoms
from ase.lattice.hexagonal import Hexagonal
from gpaw import GPAW, FermiDirac
from gpaw.test import findpeak, equal
from gpaw.response.bse import BSE

if 1:
    calc = GPAW(mode='pw',
                xc='PBE',
                nbands='nao',
                setups={'Mo': '6'},
                parallel={'band': 1, 'domain': 1},
                occupations=FermiDirac(0.001),
                convergence={'bands': -5},
                kpts=(9, 9, 1))

    a = 3.1604
    c = 10.0

    cell = Hexagonal(symbol='Mo', latticeconstant={'a': a, 'c': c}).get_cell()
    layer = Atoms(symbols='MoS2', cell=cell, pbc=(1, 1, 1),
                  scaled_positions=[(0, 0, 0),
                                    (2 / 3, 1 / 3, 0.3),
                                    (2 / 3, 1 / 3, -0.3)])

    pos = layer.get_positions()
    pos[1][2] = pos[0][2] + 3.172 / 2
    pos[2][2] = pos[0][2] - 3.172 / 2
    layer.set_positions(pos)
    layer.set_calculator(calc)
    layer.get_potential_energy()
    calc.write('MoS2.gpw', mode='all')

bse = BSE('MoS2.gpw',
          spinors=True,
          ecut=10,
          valence_bands=[8],
          conduction_bands=[9],
          eshift=0.8,
          nbands=15,
          write_h=False,
          write_v=False,
          wfile=None,
          mode='BSE',
          truncation='2D')

w_w, alpha_w = bse.get_polarizability(filename=None,
                                      pbc=[True, True, False],
                                      write_eig=None,
                                      eta=0.02,
                                      w_w=np.linspace(0., 5., 5001))

w0, I0 = findpeak(w_w[:1100], alpha_w.imag[:1100])
w1, I1 = findpeak(w_w[1100:1300], alpha_w.imag[1100:1300])
w1 += 1.1
equal(w0, 1.01, 0.01)
equal(I0, 13.65, 0.35)
equal(w1, 1.16, 0.01)
equal(I1, 12.6, 0.35)



import numpy as np
from ase.units import Bohr

from gpaw.lfc import BasisFunctions
from gpaw.utilities import unpack
from gpaw.utilities.tools import tri2full
#from gpaw import debug
#from gpaw.lcao.overlap import NewTwoCenterIntegrals as NewTCI
from gpaw.lcao.tci import TCIExpansions
from gpaw.utilities.blas import gemm, gemmdot
from gpaw.wavefunctions.base import WaveFunctions
from gpaw.lcao.atomic_correction import (DenseAtomicCorrection,
                                         SparseAtomicCorrection)
from gpaw.wavefunctions.mode import Mode


class LCAO(Mode):
    name = 'lcao'

    def __init__(self, atomic_correction=None, interpolation=3,
                 force_complex_dtype=False):
        self.atomic_correction = atomic_correction
        self.interpolation = interpolation
        Mode.__init__(self, force_complex_dtype)

    def __call__(self, *args, **kwargs):
        return LCAOWaveFunctions(*args,
                                 atomic_correction=self.atomic_correction,
                                 **kwargs)

    def __repr__(self):
        return 'LCAO({})'.format(self.todict())

    def todict(self):
        dct = Mode.todict(self)
        dct['interpolation'] = self.interpolation
        return dct


def update_phases(C_unM, q_u, ibzk_qc, spos_ac, oldspos_ac, setups, Mstart):
    """Complex-rotate coefficients compensating discontinuous phase shift.

    This changes the coefficients to counteract the phase discontinuity
    of overlaps when atoms move across a cell boundary."""

    # We don't want to apply any phase shift unless we crossed a cell
    # boundary.  So we round the shift to either 0 or 1.
    #
    # Example: spos_ac goes from 0.01 to 0.99 -- this rounds to 1 and
    # we apply the phase.  If someone moves an atom by half a cell
    # without crossing a boundary, then we are out of luck.  But they
    # should have reinitialized from LCAO anyway.
    phase_qa = np.exp(2j * np.pi *
                      np.dot(ibzk_qc, (spos_ac - oldspos_ac).T.round()))

    for q, C_nM in zip(q_u, C_unM):
        if C_nM is None:
            continue
        for a in range(len(spos_ac)):
            M1 = setups.M_a[a] - Mstart
            M2 = M1 + setups[a].nao
            M1 = max(0, M1)
            C_nM[:, M1:M2] *= phase_qa[q, a]  # (may truncate M2)


# replace by class to make data structure perhaps a bit less confusing
def get_r_and_offsets(nl, spos_ac, cell_cv):
    r_and_offset_aao = {}

    def add(a1, a2, R_c, offset):
        if not (a1, a2) in r_and_offset_aao:
            r_and_offset_aao[(a1, a2)] = []
        r_and_offset_aao[(a1, a2)].append((R_c, offset))

    for a1, spos1_c in enumerate(spos_ac):
        a2_a, offsets = nl.get_neighbors(a1)
        for a2, offset in zip(a2_a, offsets):
            spos2_c = spos_ac[a2] + offset

            R_c = np.dot(spos2_c - spos1_c, cell_cv)
            add(a1, a2, R_c, offset)
            if a1 != a2 or offset.any():
                add(a2, a1, -R_c, -offset)

    return r_and_offset_aao


class LCAOWaveFunctions(WaveFunctions):
    mode = 'lcao'

    def __init__(self, ksl, gd, nvalence, setups, bd,
                 dtype, world, kd, kptband_comm, timer,
                 atomic_correction=None, collinear=True):
        WaveFunctions.__init__(self, gd, nvalence, setups, bd,
                               dtype, collinear, world, kd,
                               kptband_comm, timer)
        self.ksl = ksl
        self.S_qMM = None
        self.T_qMM = None
        self.P_aqMi = None
        self.debug_tci = False

        if atomic_correction is None:
            atomic_correction = 'sparse' if ksl.using_blacs else 'dense'

        if atomic_correction == 'sparse':
            self.atomic_correction_cls = SparseAtomicCorrection
        else:
            assert atomic_correction == 'dense'
            self.atomic_correction_cls = DenseAtomicCorrection

        #self.tci = NewTCI(gd.cell_cv, gd.pbc_c, setups, kd.ibzk_qc, kd.gamma)
        with self.timer('TCI: Evaluate splines'):
            self.tciexpansions = TCIExpansions.new_from_setups(setups)

        self.basis_functions = BasisFunctions(gd,
                                              [setup.phit_j
                                               for setup in setups],
                                              kd,
                                              dtype=dtype,
                                              cut=True)

    def set_orthonormalized(self, o):
        pass

    def empty(self, n=(), global_array=False, realspace=False):
        if realspace:
            return self.gd.empty(n, self.dtype, global_array)
        else:
            if isinstance(n, int):
                n = (n,)
            nao = self.setups.nao
            return np.empty(n + (nao,), self.dtype)

    def __str__(self):
        s = 'Wave functions: LCAO\n'
        s += '  Diagonalizer: %s\n' % self.ksl.get_description()
        s += ('  Atomic Correction: %s\n'
              % self.atomic_correction_cls.description)
        s += '  Datatype: %s\n' % self.dtype.__name__
        return s

    def set_eigensolver(self, eigensolver):
        WaveFunctions.set_eigensolver(self, eigensolver)
        if eigensolver:
            eigensolver.initialize(self.gd, self.dtype, self.setups.nao,
                                   self.ksl)

    def set_positions(self, spos_ac, atom_partition=None, move_wfs=False):
        oldspos_ac = self.spos_ac
        with self.timer('Basic WFS set positions'):
            WaveFunctions.set_positions(self, spos_ac, atom_partition)

        with self.timer('Basis functions set positions'):
            self.basis_functions.set_positions(spos_ac)

        if self.ksl is not None:
            self.basis_functions.set_matrix_distribution(self.ksl.Mstart,
                                                         self.ksl.Mstop)

        nq = len(self.kd.ibzk_qc)
        nao = self.setups.nao
        mynbands = self.bd.mynbands
        Mstop = self.ksl.Mstop
        Mstart = self.ksl.Mstart
        mynao = Mstop - Mstart

        #if self.ksl.using_blacs:  # XXX
        #     S and T have been distributed to a layout with blacs, so
        #     discard them to force reallocation from scratch.
        #
        #     TODO: evaluate S and T when they *are* distributed, thus saving
        #     memory and avoiding this problem
        for kpt in self.kpt_u:
            kpt.S_MM = None
            kpt.T_MM = None

        # Free memory in case of old matrices:
        self.S_qMM = self.T_qMM = self.P_aqMi = None

        if self.dtype == complex and oldspos_ac is not None:
            update_phases([kpt.C_nM for kpt in self.kpt_u],
                          [kpt.q for kpt in self.kpt_u],
                          self.kd.ibzk_qc, spos_ac, oldspos_ac,
                          self.setups, Mstart)

        for kpt in self.kpt_u:
            if kpt.C_nM is None:
                kpt.C_nM = np.empty((mynbands, nao), self.dtype)

        if 0:#self.debug_tci:
            #if self.ksl.using_blacs:
            #    self.tci.set_matrix_distribution(Mstart, mynao)
            oldS_qMM = np.empty((nq, mynao, nao), self.dtype)
            oldT_qMM = np.empty((nq, mynao, nao), self.dtype)


            oldP_aqMi = {}
            for a in self.basis_functions.my_atom_indices:
                ni = self.setups[a].ni
                oldP_aqMi[a] = np.empty((nq, nao, ni), self.dtype)

            # Calculate lower triangle of S and T matrices:
            self.timer.start('tci calculate')
            #self.tci.calculate(spos_ac, oldS_qMM, oldT_qMM,
            #                   oldP_aqMi)
            self.timer.stop('tci calculate')


        self.timer.start('mktci')
        manytci = self.tciexpansions.get_manytci_calculator(
            self.setups, self.gd, spos_ac, self.kd.ibzk_qc, self.dtype,
            self.timer)
        self.timer.stop('mktci')
        self.manytci = manytci
        self.newtci = manytci.tci

        my_atom_indices = self.basis_functions.my_atom_indices
        self.timer.start('ST tci')
        newS_qMM, newT_qMM = manytci.O_qMM_T_qMM(self.gd.comm,
                                                 Mstart, Mstop,
                                                 self.ksl.using_blacs)
        self.timer.stop('ST tci')
        self.timer.start('P tci')
        P_qIM = manytci.P_qIM(my_atom_indices)
        self.timer.stop('P tci')
        self.P_aqMi = newP_aqMi = manytci.P_aqMi(my_atom_indices)
        self.P_qIM = P_qIM  # XXX atomic correction

        self.atomic_correction = self.atomic_correction_cls.new_from_wfs(self)

        # TODO
        #   OK complex/conj, periodic images
        #   OK scalapack
        #   derivatives/forces
        #   sparse
        #   use symmetry/conj tricks to reduce calculations
        #   enable caching of spherical harmonics

        #if self.atomic_correction.name != 'dense':
        #from gpaw.lcao.newoverlap import newoverlap
        #self.P_neighbors_a, self.P_aaqim = newoverlap(self, spos_ac)

        #if self.atomic_correction.name == 'scipy':
        #    Pold_qIM = self.atomic_correction.Psparse_qIM
        #    for q in range(nq):
        #        maxerr = abs(Pold_qIM[q] - P_qIM[q]).max()
        #        print('sparse maxerr', maxerr)
        #        assert maxerr == 0

        self.atomic_correction.add_overlap_correction(newS_qMM)
        if self.debug_tci:
            self.atomic_correction.add_overlap_correction(oldS_qMM)

        self.allocate_arrays_for_projections(my_atom_indices)

        #S_MM = None  # allow garbage collection of old S_qMM after redist
        if self.debug_tci:
            oldS_qMM = self.ksl.distribute_overlap_matrix(oldS_qMM, root=-1)
            oldT_qMM = self.ksl.distribute_overlap_matrix(oldT_qMM, root=-1)

        newS_qMM = self.ksl.distribute_overlap_matrix(newS_qMM, root=-1)
        newT_qMM = self.ksl.distribute_overlap_matrix(newT_qMM, root=-1)

        #if (debug and self.bd.comm.size == 1 and self.gd.comm.rank == 0 and
        #    nao > 0 and not self.ksl.using_blacs):
        #    S and T are summed only on comm master, so check only there
        #    from numpy.linalg import eigvalsh
        #    self.timer.start('Check positive definiteness')
        #    for S_MM in S_qMM:
        #        tri2full(S_MM, UL='L')
        #        smin = eigvalsh(S_MM).real.min()
        #        if smin < 0:
        #            raise RuntimeError('Overlap matrix has negative '
        #                               'eigenvalue: %e' % smin)
        #    self.timer.stop('Check positive definiteness')
        self.positions_set = True

        if self.debug_tci:
            Serr = np.abs(newS_qMM - oldS_qMM).max()
            Terr = np.abs(newT_qMM - oldT_qMM).max()
            print('S maxerr', Serr)
            print('T maxerr', Terr)
            try:
                assert Terr < 1e-15, Terr
            except AssertionError:
                np.set_printoptions(precision=6)
                if self.world.rank == 0:
                    print(newT_qMM)
                    print(oldT_qMM)
                    print(newT_qMM - oldT_qMM)
                raise
            assert Serr < 1e-15, Serr

            assert len(oldP_aqMi) == len(newP_aqMi)
            for a in oldP_aqMi:
                Perr = np.abs(oldP_aqMi[a] - newP_aqMi[a]).max()
                assert Perr < 1e-15, (a, Perr)

        for kpt in self.kpt_u:
            q = kpt.q
            kpt.S_MM = newS_qMM[q]
            kpt.T_MM = newT_qMM[q]
        self.S_qMM = newS_qMM
        self.T_qMM = newT_qMM

        # Elpa wants to reuse the decomposed form of S_qMM.
        # We need to keep track of the existence of that object here,
        # since this is where we change S_qMM.  Hence, expect this to
        # become arrays after the first diagonalization:
        self.decomposed_S_qMM = [None] * len(self.S_qMM)

    def initialize(self, density, hamiltonian, spos_ac):
        # Note: The above line exists also in set_positions.
        # This is guaranteed to be correct, but we can probably remove one.
        # Of course no human can understand the initialization process,
        # so this will be some other day.
        self.timer.start('LCAO WFS Initialize')
        if density.nt_sG is None:
            if self.kpt_u[0].f_n is None or self.kpt_u[0].C_nM is None:
                density.initialize_from_atomic_densities(self.basis_functions)
            else:
                # We have the info we need for a density matrix, so initialize
                # from that instead of from scratch.  This will be the case
                # after set_positions() during a relaxation
                density.initialize_from_wavefunctions(self)
            # Initialize GLLB-potential from basis function orbitals
            if hamiltonian.xc.type == 'GLLB':
                hamiltonian.xc.initialize_from_atomic_orbitals(
                    self.basis_functions)

        else:
            # After a restart, nt_sg doesn't exist yet, so we'll have to
            # make sure it does.  Of course, this should have been taken care
            # of already by this time, so we should improve the code elsewhere
            density.calculate_normalized_charges_and_mix()

        hamiltonian.update(density)
        self.timer.stop('LCAO WFS Initialize')

        return 0, 0

    def initialize_wave_functions_from_lcao(self):
        """Fill the calc.wfs.kpt_[u].psit_nG arrays with useful data.

        Normally psit_nG is NOT used in lcao mode, but some extensions
        (like ase.dft.wannier) want to have it.
        This code is adapted from fd.py / initialize_from_lcao_coefficients()
        and fills psit_nG with data constructed from the current lcao
        coefficients (kpt.C_nM).

        (This may or may not work in band-parallel case!)
        """
        from gpaw.wavefunctions.arrays import UniformGridWaveFunctions
        bfs = self.basis_functions
        for kpt in self.mykpts:
            kpt.psit = UniformGridWaveFunctions(
                self.bd.nbands, self.gd, self.dtype, kpt=kpt.q, dist=None,
                spin=kpt.s, collinear=True)
            kpt.psit_nG[:] = 0.0
            bfs.lcao_to_grid(kpt.C_nM, kpt.psit_nG[:self.bd.mynbands], kpt.q)

    def initialize_wave_functions_from_restart_file(self):
        """Dummy function to ensure compatibility to fd mode"""
        self.initialize_wave_functions_from_lcao()

    def add_orbital_density(self, nt_G, kpt, n):
        rank, u = self.kd.get_rank_and_index(kpt.s, kpt.k)
        assert rank == self.kd.comm.rank
        assert self.kpt_u[u] is kpt
        psit_G = self._get_wave_function_array(u, n, realspace=True)
        self.add_realspace_orbital_to_density(nt_G, psit_G)

    def calculate_density_matrix(self, f_n, C_nM, rho_MM=None):
        self.timer.start('Calculate density matrix')
        rho_MM = self.ksl.calculate_density_matrix(f_n, C_nM, rho_MM)
        self.timer.stop('Calculate density matrix')
        return rho_MM

        if 1:
            # XXX Should not conjugate, but call gemm(..., 'c')
            # Although that requires knowing C_Mn and not C_nM.
            # that also conforms better to the usual conventions in literature
            Cf_Mn = C_nM.T.conj() * f_n
            self.timer.start('gemm')
            gemm(1.0, C_nM, Cf_Mn, 0.0, rho_MM, 'n')
            self.timer.stop('gemm')
            self.timer.start('band comm sum')
            self.bd.comm.sum(rho_MM)
            self.timer.stop('band comm sum')
        else:
            # Alternative suggestion. Might be faster. Someone should test this
            from gpaw.utilities.blas import r2k
            C_Mn = C_nM.T.copy()
            r2k(0.5, C_Mn, f_n * C_Mn, 0.0, rho_MM)
            tri2full(rho_MM)

    def calculate_atomic_density_matrices_with_occupation(self, D_asp, f_un):
        #ac = self.atomic_correction
        #if ac.implements_distributed_projections():
        #    D2_asp = ac.redistribute(self, D_asp, type='asp', op='forth')
        #    WaveFunctions.calculate_atomic_density_matrices_with_occupation(
        #        self, D2_asp, f_un)
        #    D3_asp = ac.redistribute(self, D2_asp, type='asp', op='back')
        #    for a in D_asp:
        #        D_asp[a][:] = D3_asp[a]
        #else:
        WaveFunctions.calculate_atomic_density_matrices_with_occupation(
            self, D_asp, f_un)

    def calculate_density_matrix_delta(self, d_nn, C_nM, rho_MM=None):
        self.timer.start('Calculate density matrix')
        rho_MM = self.ksl.calculate_density_matrix_delta(d_nn, C_nM, rho_MM)
        self.timer.stop('Calculate density matrix')
        return rho_MM

    def add_to_density_from_k_point_with_occupation(self, nt_sG, kpt, f_n):
        """Add contribution to pseudo electron-density. Do not use the standard
        occupation numbers, but ones given with argument f_n."""
        # Custom occupations are used in calculation of response potential
        # with GLLB-potential
        if kpt.rho_MM is None:
            rho_MM = self.calculate_density_matrix(f_n, kpt.C_nM)
            if hasattr(kpt, 'c_on'):
                assert self.bd.comm.size == 1
                d_nn = np.zeros((self.bd.mynbands, self.bd.mynbands),
                                dtype=kpt.C_nM.dtype)
                for ne, c_n in zip(kpt.ne_o, kpt.c_on):
                    assert abs(c_n.imag).max() < 1e-14
                    d_nn += ne * np.outer(c_n.conj(), c_n).real
                rho_MM += self.calculate_density_matrix_delta(d_nn, kpt.C_nM)
        else:
            rho_MM = kpt.rho_MM
        self.timer.start('Construct density')
        self.basis_functions.construct_density(rho_MM, nt_sG[kpt.s], kpt.q)
        self.timer.stop('Construct density')

    def add_to_kinetic_density_from_k_point(self, taut_G, kpt):
        raise NotImplementedError('Kinetic density calculation for LCAO '
                                  'wavefunctions is not implemented.')

    def calculate_forces(self, hamiltonian, F_av):
        self.timer.start('LCAO forces')

        ksl = self.ksl
        nao = ksl.nao
        mynao = ksl.mynao
        dtype = self.dtype
        #tci = self.tci
        newtci = self.newtci
        gd = self.gd
        bfs = self.basis_functions

        Mstart = ksl.Mstart
        Mstop = ksl.Mstop

        from gpaw.kohnsham_layouts import BlacsOrbitalLayouts
        isblacs = isinstance(ksl, BlacsOrbitalLayouts)  # XXX

        if not isblacs:
            self.timer.start('TCI derivative')

            dThetadR_qvMM, dTdR_qvMM = self.manytci.O_qMM_T_qMM(
                gd.comm, Mstart, Mstop, False, derivative=True)

            dPdR_aqvMi = self.manytci.P_aqMi(
                self.basis_functions.my_atom_indices, derivative=True)

            gd.comm.sum(dThetadR_qvMM)
            gd.comm.sum(dTdR_qvMM)
            self.timer.stop('TCI derivative')

            my_atom_indices = bfs.my_atom_indices
            atom_indices = bfs.atom_indices

            def _slices(indices):
                for a in indices:
                    M1 = bfs.M_a[a] - Mstart
                    M2 = M1 + self.setups[a].nao
                    if M2 > 0:
                        yield a, max(0, M1), M2

            def slices():
                return _slices(atom_indices)

            def my_slices():
                return _slices(my_atom_indices)

        dH_asp = hamiltonian.dH_asp
        vt_sG = hamiltonian.vt_sG

        #
        #         -----                    -----
        #          \    -1                  \    *
        # E      =  )  S     H    rho     =  )  c     eps  f  c
        #  mu nu   /    mu x  x z    z nu   /    n mu    n  n  n nu
        #         -----                    -----
        #          x z                       n
        #
        # We use the transpose of that matrix.  The first form is used
        # if rho is given, otherwise the coefficients are used.
        self.timer.start('Initial')

        rhoT_uMM = []
        ET_uMM = []

        if not isblacs:
            if self.kpt_u[0].rho_MM is None:
                self.timer.start('Get density matrix')
                for kpt in self.kpt_u:
                    rhoT_MM = ksl.get_transposed_density_matrix(kpt.f_n,
                                                                kpt.C_nM)
                    rhoT_uMM.append(rhoT_MM)
                    ET_MM = ksl.get_transposed_density_matrix(kpt.f_n *
                                                              kpt.eps_n,
                                                              kpt.C_nM)
                    ET_uMM.append(ET_MM)

                    if hasattr(kpt, 'c_on'):
                        # XXX does this work with BLACS/non-BLACS/etc.?
                        assert self.bd.comm.size == 1
                        d_nn = np.zeros((self.bd.mynbands, self.bd.mynbands),
                                        dtype=kpt.C_nM.dtype)
                        for ne, c_n in zip(kpt.ne_o, kpt.c_on):
                            d_nn += ne * np.outer(c_n.conj(), c_n)
                        rhoT_MM += ksl.get_transposed_density_matrix_delta(
                            d_nn, kpt.C_nM)
                        ET_MM += ksl.get_transposed_density_matrix_delta(
                            d_nn * kpt.eps_n, kpt.C_nM)
                self.timer.stop('Get density matrix')
            else:
                rhoT_uMM = []
                ET_uMM = []
                for kpt in self.kpt_u:
                    H_MM = self.eigensolver.calculate_hamiltonian_matrix(
                        hamiltonian, self, kpt)
                    tri2full(H_MM)
                    S_MM = kpt.S_MM.copy()
                    tri2full(S_MM)
                    ET_MM = np.linalg.solve(S_MM, gemmdot(H_MM,
                                                          kpt.rho_MM)).T.copy()
                    del S_MM, H_MM
                    rhoT_MM = kpt.rho_MM.T.copy()
                    rhoT_uMM.append(rhoT_MM)
                    ET_uMM.append(ET_MM)
        self.timer.stop('Initial')

        if isblacs:  # XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
            from gpaw.blacs import BlacsGrid, Redistributor

            def get_density_matrix(f_n, C_nM, redistributor):
                rho1_mm = ksl.calculate_blocked_density_matrix(f_n,
                                                               C_nM).conj()
                rho_mm = redistributor.redistribute(rho1_mm)
                return rho_mm

            #pcutoff_a = [max([pt.get_cutoff() for pt in setup.pt_j])
            #             for setup in self.setups]
            #phicutoff_a = [max([phit.get_cutoff() for phit in setup.phit_j])
            #               for setup in self.setups]

            # XXX should probably use bdsize x gdsize instead
            # That would be consistent with some existing grids
            grid = BlacsGrid(ksl.block_comm, self.gd.comm.size,
                             self.bd.comm.size)

            blocksize1 = -(-nao // grid.nprow)
            blocksize2 = -(-nao // grid.npcol)
            # XXX what are rows and columns actually?
            desc = grid.new_descriptor(nao, nao, blocksize1, blocksize2)

            rhoT_umm = []
            ET_umm = []
            redistributor = Redistributor(grid.comm, ksl.mmdescriptor, desc)
            Fpot_av = np.zeros_like(F_av)
            for u, kpt in enumerate(self.kpt_u):
                self.timer.start('Get density matrix')
                rhoT_mm = get_density_matrix(kpt.f_n, kpt.C_nM, redistributor)
                rhoT_umm.append(rhoT_mm)
                self.timer.stop('Get density matrix')

                self.timer.start('Potential')
                rhoT_mM = ksl.distribute_to_columns(rhoT_mm, desc)

                vt_G = vt_sG[kpt.s]
                Fpot_av += bfs.calculate_force_contribution(vt_G, rhoT_mM,
                                                            kpt.q)
                del rhoT_mM
                self.timer.stop('Potential')

            self.timer.start('Get density matrix')
            for kpt in self.kpt_u:
                ET_mm = get_density_matrix(kpt.f_n * kpt.eps_n, kpt.C_nM,
                                           redistributor)
                ET_umm.append(ET_mm)
            self.timer.stop('Get density matrix')

            M1start = blocksize1 * grid.myrow
            M2start = blocksize2 * grid.mycol

            M1stop = min(M1start + blocksize1, nao)
            M2stop = min(M2start + blocksize2, nao)

            m1max = M1stop - M1start
            m2max = M2stop - M2start

        if not isblacs:
            # Kinetic energy contribution
            #
            #           ----- d T
            #  a         \       mu nu
            # F += 2 Re   )   -------- rho
            #            /    d R         nu mu
            #           -----    mu nu
            #        mu in a; nu
            #
            Fkin_av = np.zeros_like(F_av)
            for u, kpt in enumerate(self.kpt_u):
                dEdTrhoT_vMM = (dTdR_qvMM[kpt.q] *
                                rhoT_uMM[u][np.newaxis]).real
                # XXX load distribution!
                for a, M1, M2 in my_slices():
                    Fkin_av[a, :] += \
                        2.0 * dEdTrhoT_vMM[:, M1:M2].sum(-1).sum(-1)
            del dEdTrhoT_vMM

            # Density matrix contribution due to basis overlap
            #
            #            ----- d Theta
            #  a          \           mu nu
            # F  += -2 Re  )   ------------  E
            #             /        d R        nu mu
            #            -----        mu nu
            #         mu in a; nu
            #
            Ftheta_av = np.zeros_like(F_av)
            for u, kpt in enumerate(self.kpt_u):
                dThetadRE_vMM = (dThetadR_qvMM[kpt.q] *
                                 ET_uMM[u][np.newaxis]).real
                for a, M1, M2 in my_slices():
                    Ftheta_av[a, :] += \
                        -2.0 * dThetadRE_vMM[:, M1:M2].sum(-1).sum(-1)
            del dThetadRE_vMM

        if isblacs:
            #from gpaw.lcao.overlap import TwoCenterIntegralCalculator
            self.timer.start('Prepare TCI loop')
            M_a = bfs.M_a
            Fkin2_av = np.zeros_like(F_av)
            Ftheta2_av = np.zeros_like(F_av)
            atompairs = self.newtci.a1a2.get_atompairs()

            self.timer.start('broadcast dH')
            alldH_asp = {}
            for a in range(len(self.setups)):
                gdrank = bfs.sphere_a[a].rank
                if gdrank == gd.rank:
                    dH_sp = dH_asp[a]
                else:
                    ni = self.setups[a].ni
                    dH_sp = np.empty((self.nspins, ni * (ni + 1) // 2))
                gd.comm.broadcast(dH_sp, gdrank)
                # okay, now everyone gets copies of dH_sp
                alldH_asp[a] = dH_sp
            self.timer.stop('broadcast dH')

            # This will get sort of hairy.  We need to account for some
            # three-center overlaps, such as:
            #
            #         a1
            #      Phi   ~a3    a3  ~a3     a2     a2,a1
            #   < ----  |p  > dH   <p   |Phi  > rho
            #      dR
            #
            # To this end we will loop over all pairs of atoms (a1, a3),
            # and then a sub-loop over (a3, a2).

            self.timer.stop('Prepare TCI loop')
            self.timer.start('Not so complicated loop')

            for (a1, a2) in atompairs:
                if a1 >= a2:
                    # Actually this leads to bad load balance.
                    # We should take a1 > a2 or a1 < a2 equally many times.
                    # Maybe decide which of these choices
                    # depending on whether a2 % 1 == 0
                    continue

                m1start = M_a[a1] - M1start
                m2start = M_a[a2] - M2start
                if m1start >= blocksize1 or m2start >= blocksize2:
                    continue  # (we have only one block per CPU)

                nm1 = self.setups[a1].nao
                nm2 = self.setups[a2].nao

                m1stop = min(m1start + nm1, m1max)
                m2stop = min(m2start + nm2, m2max)

                if m1stop <= 0 or m2stop <= 0:
                    continue

                m1start = max(m1start, 0)
                m2start = max(m2start, 0)
                J1start = max(0, M1start - M_a[a1])
                J2start = max(0, M2start - M_a[a2])
                M1stop = J1start + m1stop - m1start
                J2stop = J2start + m2stop - m2start

                dThetadR_qvmm, dTdR_qvmm = newtci.dOdR_dTdR(a1, a2)

                for u, kpt in enumerate(self.kpt_u):
                    rhoT_mm = rhoT_umm[u][m1start:m1stop, m2start:m2stop]
                    ET_mm = ET_umm[u][m1start:m1stop, m2start:m2stop]
                    Fkin_v = 2.0 * (dTdR_qvmm[kpt.q][:, J1start:M1stop,
                                                     J2start:J2stop] *
                                    rhoT_mm[np.newaxis]).real.sum(-1).sum(-1)
                    Ftheta_v = 2.0 * (dThetadR_qvmm[kpt.q][:, J1start:M1stop,
                                                           J2start:J2stop] *
                                      ET_mm[np.newaxis]).real.sum(-1).sum(-1)
                    Fkin2_av[a1] += Fkin_v
                    Fkin2_av[a2] -= Fkin_v
                    Ftheta2_av[a1] -= Ftheta_v
                    Ftheta2_av[a2] += Ftheta_v

            Fkin_av = Fkin2_av
            Ftheta_av = Ftheta2_av
            self.timer.stop('Not so complicated loop')

            dHP_and_dSP_aauim = {}

            a2values = {}
            for (a2, a3) in atompairs:
                if a3 not in a2values:
                    a2values[a3] = []
                a2values[a3].append(a2)

            Fatom_av = np.zeros_like(F_av)
            Frho_av = np.zeros_like(F_av)
            self.timer.start('Complicated loop')
            for a1, a3 in atompairs:
                if a1 == a3:
                    # Functions reside on same atom, so their overlap
                    # does not change when atom is displaced
                    continue
                m1start = M_a[a1] - M1start
                if m1start >= blocksize1:
                    continue

                nm1 = self.setups[a1].nao
                m1stop = min(m1start + nm1, m1max)
                if m1stop <= 0:
                    continue

                dPdR_qvim = newtci.dPdR(a3, a1)
                if dPdR_qvim is None:
                    continue

                dPdR_qvmi = -dPdR_qvim.transpose(0, 1, 3, 2).conj()

                m1start = max(m1start, 0)
                J1start = max(0, M1start - M_a[a1])
                J1stop = J1start + m1stop - m1start
                dPdR_qvmi = dPdR_qvmi[:, :, J1start:J1stop, :].copy()
                for a2 in a2values[a3]:
                    m2start = M_a[a2] - M2start
                    if m2start >= blocksize2:
                        continue

                    nm2 = self.setups[a2].nao
                    m2stop = min(m2start + nm2, m2max)
                    if m2stop <= 0:
                        continue

                    m2start = max(m2start, 0)
                    J2start = max(0, M2start - M_a[a2])
                    J2stop = J2start + m2stop - m2start

                    if (a2, a3) in dHP_and_dSP_aauim:
                        dHP_uim, dSP_uim = dHP_and_dSP_aauim[(a2, a3)]
                    else:
                        P_qim = newtci.P(a3, a2)
                        if P_qim is None:
                            continue
                        P_qmi = P_qim.transpose(0, 2, 1).conj()
                        P_qmi = P_qmi[:, J2start:J2stop].copy()
                        dH_sp = alldH_asp[a3]
                        dS_ii = self.setups[a3].dO_ii

                        dHP_uim = []
                        dSP_uim = []
                        for u, kpt in enumerate(self.kpt_u):
                            dH_ii = unpack(dH_sp[kpt.s])
                            dHP_im = np.dot(P_qmi[kpt.q], dH_ii).T.conj()
                            # XXX only need nq of these,
                            # but the looping is over all u
                            dSP_im = np.dot(P_qmi[kpt.q], dS_ii).T.conj()
                            dHP_uim.append(dHP_im)
                            dSP_uim.append(dSP_im)
                            dHP_and_dSP_aauim[(a2, a3)] = dHP_uim, dSP_uim

                    for u, kpt in enumerate(self.kpt_u):
                        rhoT_mm = rhoT_umm[u][m1start:m1stop, m2start:m2stop]
                        ET_mm = ET_umm[u][m1start:m1stop, m2start:m2stop]
                        dPdRdHP_vmm = np.dot(dPdR_qvmi[kpt.q], dHP_uim[u])
                        dPdRdSP_vmm = np.dot(dPdR_qvmi[kpt.q], dSP_uim[u])

                        Fatom_c = 2.0 * (dPdRdHP_vmm *
                                         rhoT_mm).real.sum(-1).sum(-1)
                        Frho_c = 2.0 * (dPdRdSP_vmm *
                                        ET_mm).real.sum(-1).sum(-1)
                        Fatom_av[a1] += Fatom_c
                        Fatom_av[a3] -= Fatom_c

                        Frho_av[a1] -= Frho_c
                        Frho_av[a3] += Frho_c

            self.timer.stop('Complicated loop')

        if not isblacs:
            # Potential contribution
            #
            #           -----      /  d Phi  (r)
            #  a         \        |        mu    ~
            # F += -2 Re  )       |   ---------- v (r)  Phi  (r) dr rho
            #            /        |     d R                nu          nu mu
            #           -----    /         a
            #        mu in a; nu
            #
            self.timer.start('Potential')
            Fpot_av = np.zeros_like(F_av)

            for u, kpt in enumerate(self.kpt_u):
                vt_G = vt_sG[kpt.s]
                Fpot_av += bfs.calculate_force_contribution(vt_G, rhoT_uMM[u],
                                                            kpt.q)
            self.timer.stop('Potential')

            # Density matrix contribution from PAW correction
            #
            #           -----                        -----
            #  a         \      a                     \     b
            # F +=  2 Re  )    Z      E        - 2 Re  )   Z      E
            #            /      mu nu  nu mu          /     mu nu  nu mu
            #           -----                        -----
            #           mu nu                    b; mu in a; nu
            #
            # with
            #                  b*
            #         -----  dP
            #   b      \       i mu    b   b
            #  Z     =  )   -------- dS   P
            #   mu nu  /     dR        ij  j nu
            #         -----    b mu
            #           ij
            #
            self.timer.start('Paw correction')
            Frho_av = np.zeros_like(F_av)
            for u, kpt in enumerate(self.kpt_u):
                work_MM = np.zeros((mynao, nao), dtype)
                ZE_MM = None
                for b in my_atom_indices:
                    setup = self.setups[b]
                    dO_ii = np.asarray(setup.dO_ii, dtype)
                    dOP_iM = np.zeros((setup.ni, nao), dtype)
                    gemm(1.0, self.P_aqMi[b][kpt.q], dO_ii, 0.0, dOP_iM, 'c')
                    for v in range(3):
                        gemm(1.0, dOP_iM,
                             dPdR_aqvMi[b][kpt.q][v][Mstart:Mstop],
                             0.0, work_MM, 'n')
                        ZE_MM = (work_MM * ET_uMM[u]).real
                        for a, M1, M2 in slices():
                            dE = 2 * ZE_MM[M1:M2].sum()
                            Frho_av[a, v] -= dE  # the "b; mu in a; nu" term
                            Frho_av[b, v] += dE  # the "mu nu" term
            del work_MM, ZE_MM
            self.timer.stop('Paw correction')

            # Atomic density contribution
            #            -----                         -----
            #  a          \     a                       \     b
            # F  += -2 Re  )   A      rho       + 2 Re   )   A      rho
            #             /     mu nu    nu mu          /     mu nu    nu mu
            #            -----                         -----
            #            mu nu                     b; mu in a; nu
            #
            #                  b*
            #         ----- d P
            #  b       \       i mu   b   b
            # A     =   )   ------- dH   P
            #  mu nu   /    d R       ij  j nu
            #         -----    b mu
            #           ij
            #
            self.timer.start('Atomic Hamiltonian force')
            Fatom_av = np.zeros_like(F_av)
            for u, kpt in enumerate(self.kpt_u):
                for b in my_atom_indices:
                    H_ii = np.asarray(unpack(dH_asp[b][kpt.s]), dtype)
                    HP_iM = gemmdot(H_ii,
                                    np.ascontiguousarray(
                                        self.P_aqMi[b][kpt.q].T.conj()))
                    for v in range(3):
                        dPdR_Mi = dPdR_aqvMi[b][kpt.q][v][Mstart:Mstop]
                        ArhoT_MM = (gemmdot(dPdR_Mi, HP_iM) * rhoT_uMM[u]).real
                        for a, M1, M2 in slices():
                            dE = 2 * ArhoT_MM[M1:M2].sum()
                            Fatom_av[a, v] += dE  # the "b; mu in a; nu" term
                            Fatom_av[b, v] -= dE  # the "mu nu" term
            self.timer.stop('Atomic Hamiltonian force')

        F_av += Fkin_av + Fpot_av + Ftheta_av + Frho_av + Fatom_av
        self.timer.start('Wait for sum')
        ksl.orbital_comm.sum(F_av)
        if self.bd.comm.rank == 0:
            self.kd.comm.sum(F_av, 0)
        self.timer.stop('Wait for sum')
        self.timer.stop('LCAO forces')

    def _get_wave_function_array(self, u, n, realspace=True, periodic=False):
        # XXX Taking kpt is better than taking u
        kpt = self.kpt_u[u]
        C_M = kpt.C_nM[n]

        if realspace:
            psit_G = self.gd.zeros(dtype=self.dtype)
            self.basis_functions.lcao_to_grid(C_M, psit_G, kpt.q)
            if periodic and self.dtype == complex:
                k_c = self.kd.ibzk_kc[kpt.k]
                return self.gd.plane_wave(-k_c) * psit_G
            return psit_G
        else:
            return C_M

    def write(self, writer, write_wave_functions=False):
        WaveFunctions.write(self, writer)
        if write_wave_functions:
            self.write_wave_functions(writer)

    def write_wave_functions(self, writer):
        writer.add_array(
            'coefficients',
            (self.nspins, self.kd.nibzkpts, self.bd.nbands, self.setups.nao),
            dtype=self.dtype)
        for s in range(self.nspins):
            for k in range(self.kd.nibzkpts):
                C_nM = self.collect_array('C_nM', k, s)
                writer.fill(C_nM * Bohr**-1.5)

    def read(self, reader):
        WaveFunctions.read(self, reader)
        r = reader.wave_functions
        if 'coefficients' in r:
            self.read_wave_functions(r)

    def read_wave_functions(self, reader):
        for kpt in self.kpt_u:
            C_nM = reader.proxy('coefficients', kpt.s, kpt.k)
            kpt.C_nM = self.bd.empty(self.setups.nao, dtype=self.dtype)
            for myn, C_M in enumerate(kpt.C_nM):
                n = self.bd.global_index(myn)
                # XXX number of bands could have been rounded up!
                if n >= len(C_nM):
                    break
                C_M[:] = C_nM[n] * Bohr**1.5

    def estimate_memory(self, mem):
        nq = len(self.kd.ibzk_qc)
        nao = self.setups.nao
        ni_total = sum([setup.ni for setup in self.setups])
        itemsize = mem.itemsize[self.dtype]
        mem.subnode('C [qnM]', nq * self.bd.mynbands * nao * itemsize)
        nM1, nM2 = self.ksl.get_overlap_matrix_shape()
        mem.subnode('S, T [2 x qmm]', 2 * nq * nM1 * nM2 * itemsize)
        mem.subnode('P [aqMi]', nq * nao * ni_total // self.gd.comm.size)
        #self.tci.estimate_memory(mem.subnode('TCI'))
        self.basis_functions.estimate_memory(mem.subnode('BasisFunctions'))
        self.eigensolver.estimate_memory(mem.subnode('Eigensolver'),
                                         self.dtype)



import numpy as np

from gpaw.blacs import BlacsGrid
from gpaw.blacs import Redistributor


def ranks(wfs):
    import time
    time.sleep(wfs.world.rank * 0.1)
    txt = ''
    comm_i = [wfs.world, wfs.gd.comm, wfs.kd.comm,
              wfs.bd.comm, wfs.ksl.block_comm]
    for comm in comm_i:
        txt += '%2d/%2d ' % (comm.rank, comm.size)
    return txt


def collect_uMM(wfs, a_uMM, s, k):
    return collect_wuMM(wfs, [a_uMM], 0, s, k)


def collect_wuMM(wfs, a_wuMM, w, s, k):
    # This function is based on
    # gpaw/wavefunctions/base.py: WaveFunctions.collect_auxiliary()

    dtype = a_wuMM[0][0].dtype

    ksl = wfs.ksl
    NM = ksl.nao
    kpt_rank, u = wfs.kd.get_rank_and_index(s, k)

    ksl_comm = ksl.block_comm

    if wfs.kd.comm.rank == kpt_rank:
        a_MM = a_wuMM[w][u]

        # Collect within blacs grid
        if ksl.using_blacs:
            a_mm = a_MM
            grid = BlacsGrid(ksl_comm, 1, 1)
            MM_descriptor = grid.new_descriptor(NM, NM, NM, NM)
            mm2MM = Redistributor(ksl_comm,
                                  ksl.mmdescriptor,
                                  MM_descriptor)

            a_MM = MM_descriptor.empty(dtype=dtype)
            mm2MM.redistribute(a_mm, a_MM)

        # KSL master send a_MM to the global master
        if ksl_comm.rank == 0:
            if kpt_rank == 0:
                assert wfs.world.rank == 0
                # I have it already
                return a_MM
            else:
                wfs.kd.comm.send(a_MM, 0, 2017)
                return None
    elif ksl_comm.rank == 0 and kpt_rank != 0:
        assert wfs.world.rank == 0
        a_MM = np.empty((NM, NM), dtype=dtype)
        wfs.kd.comm.receive(a_MM, kpt_rank, 2017)
        return a_MM


def distribute_MM(wfs, a_MM):
    ksl = wfs.ksl
    if not ksl.using_blacs:
        return a_MM

    dtype = a_MM.dtype
    ksl_comm = ksl.block_comm
    NM = ksl.nao
    grid = BlacsGrid(ksl_comm, 1, 1)
    MM_descriptor = grid.new_descriptor(NM, NM, NM, NM)
    MM2mm = Redistributor(ksl_comm,
                          MM_descriptor,
                          ksl.mmdescriptor)
    if ksl_comm.rank != 0:
        a_MM = MM_descriptor.empty(dtype=dtype)

    a_mm = ksl.mmdescriptor.empty(dtype=dtype)
    MM2mm.redistribute(a_MM, a_mm)
    return a_mm


def write_uMM(wfs, writer, name, a_uMM):
    return write_wuMM(wfs, writer, name, [a_uMM], wlist=[0])


def write_wuMM(wfs, writer, name, a_wuMM, wlist):
    NM = wfs.ksl.nao
    dtype = a_wuMM[0][0].dtype
    writer.add_array(name,
                     (len(wlist), wfs.nspins, wfs.kd.nibzkpts, NM, NM),
                     dtype=dtype)
    for w in wlist:
        for s in range(wfs.nspins):
            for k in range(wfs.kd.nibzkpts):
                a_MM = collect_wuMM(wfs, a_wuMM, w, s, k)
                writer.fill(a_MM)


def read_uMM(wfs, reader, name):
    return read_wuMM(wfs, reader, name, wlist=[0])[0]


def read_wuMM(wfs, reader, name, wlist):
    a_wuMM = []
    for w in wlist:
        a_uMM = []
        for kpt in wfs.kpt_u:
            indices = (w, kpt.s, kpt.k)
            # TODO: does this read on all the ksl ranks in vain?
            a_MM = reader.proxy(name, *indices)[:]
            a_MM = distribute_MM(wfs, a_MM)
            a_uMM.append(a_MM)
        a_wuMM.append(a_uMM)
    return a_wuMM



import numpy as np

from ase.build import molecule
from gpaw import GPAW
from gpaw.lcaotddft import LCAOTDDFT
from gpaw.poisson import PoissonSolver
from gpaw.lcaotddft.dipolemomentwriter import DipoleMomentWriter
from gpaw.tddft.spectrum import photoabsorption_spectrum
from gpaw.lrtddft2 import LrTDDFT2
from gpaw.mpi import world

from gpaw.test import equal

# Atoms
atoms = molecule('Na2')
atoms.center(vacuum=4.0)

# Ground-state calculation
calc = GPAW(nbands=2, h=0.4, setups=dict(Na='1'),
            basis='sz(dzp)', mode='lcao', xc='oldLDA',
            poissonsolver=PoissonSolver(eps=1e-16),
            convergence={'density': 1e-8},
            txt='gs.out')
atoms.set_calculator(calc)
energy = atoms.get_potential_energy()
calc.write('gs.gpw', mode='all')

# Time-propagation calculation
td_calc = LCAOTDDFT('gs.gpw', fxc='RPA', txt='td.out')
DipoleMomentWriter(td_calc, 'dm.dat')
td_calc.absorption_kick([0, 0, 1e-5])
td_calc.propagate(30, 150)
photoabsorption_spectrum('dm.dat', 'spec.dat',
                         e_max=10, width=0.5, delta_e=0.1)


# LrTDDFT2 calculation
calc = GPAW('gs.gpw', txt='lr.out')
lr = LrTDDFT2('lr2', calc, fxc='LDA')  # It doesn't matter which fxc is here
lr.K_matrix.fxc_pre = 0.0  # Ignore fxc part
lr.calculate()
lr.get_spectrum('lr_spec.dat', 0, 10.1, 0.1, width=0.5)
world.barrier()

# Scale spectra due to slightly different definitions
spec1_ej = np.loadtxt('spec.dat')
data1_e = spec1_ej[:, 3]
data1_e[1:] /= spec1_ej[1:, 0]
spec2_ej = np.loadtxt('lr_spec.dat')
E = lr.lr_transitions.get_transitions()[0][0]
data2_e = spec2_ej[:, 5] / E

# One can decrease the tolerance by decreasing the time step
# and other parameters
equal(data1_e, data2_e, 0.01)



from ase import Atoms
from ase.units import Hartree
from gpaw import GPAW
from gpaw.test import equal, gen
import gpaw.mpi as mpi

# Generate non-scalar-relativistic setup for Cu:
setup = gen('Cu', scalarrel=False)

a = 8.0
c = a / 2
Cu = Atoms('Cu', [(c, c, c)], magmoms=[1],
           cell=(a, a, a), pbc=0)

calc = GPAW(h=0.2,
            setups={'Cu': setup})
Cu.set_calculator(calc)
e = Cu.get_potential_energy()
niter = calc.get_number_of_iterations()

e_4s_major = calc.get_eigenvalues(spin=0)[5] / Hartree
e_3d_minor = calc.get_eigenvalues(spin=1)[4] / Hartree
print(mpi.rank, e_4s_major, e_3d_minor)

#
# The reference values are from:
#
#   http://physics.nist.gov/PhysRefData/DFTdata/Tables/29Cu.html
#
if mpi.rank == 0:
    print(e_4s_major - e_3d_minor, -0.184013 - -0.197109)
    assert abs(e_4s_major - e_3d_minor - (-0.184013 - -0.197109)) < 0.001

    print(e, niter)
    energy_tolerance = 0.0005
    niter_tolerance = 0
    equal(e, -0.271504, energy_tolerance)



from math import sqrt

from ase import Atoms

from gpaw import GPAW, Mixer, Davidson
from gpaw.test import equal
from gpaw.xc.vdw import VDWFunctional

energy_tolerance = 0.002


def test():
    vdw = VDWFunctional('vdW-DF', verbose=1)
    d = 3.9
    x = d / sqrt(3)
    L = 3.0 + 2 * 4.0
    dimer = Atoms('Ar2', [(0, 0, 0), (x, x, x)], cell=(L, L, L))
    dimer.center()
    calc = GPAW(h=0.2, xc=dict(name='revPBE', stencil=1),
                mixer=Mixer(0.8, 7, 50.0),
                eigensolver=Davidson(5))
    dimer.set_calculator(calc)
    e2 = dimer.get_potential_energy()
    calc.write('Ar2.gpw')
    e2vdw = calc.get_xc_difference(vdw)
    e2vdwb = GPAW('Ar2.gpw').get_xc_difference(vdw)
    print(e2vdwb - e2vdw)
    assert abs(e2vdwb - e2vdw) < 1e-9
    del dimer[1]
    e = dimer.get_potential_energy()
    evdw = calc.get_xc_difference(vdw)

    E = 2 * e - e2
    Evdw = E + 2 * evdw - e2vdw
    print(E, Evdw)
    assert abs(E - -0.0048) < 6e-4, abs(E)
    assert abs(Evdw - +0.0223) < 3e-3, abs(Evdw)

    print(e2, e)
    equal(e2, -0.005, energy_tolerance)
    equal(e, -0.005, energy_tolerance)


test()



import time
from math import log as ln

import numpy as np
from ase.units import Ha, Bohr

from gpaw import KohnShamConvergenceError
from gpaw.forces import calculate_forces


class SCFLoop:
    """Self-consistent field loop."""
    def __init__(self, eigenstates=0.1, energy=0.1, density=0.1, force=np.inf,
                 maxiter=100, niter_fixdensity=None, nvalence=None):
        self.max_errors = {'eigenstates': eigenstates,
                           'energy': energy,
                           'force': force,
                           'density': density}
        self.maxiter = maxiter
        self.niter_fixdensity = niter_fixdensity
        self.nvalence = nvalence

        self.old_energies = []
        self.old_F_av = None
        self.converged = False

        self.niter = None

        self.reset()

    def __str__(self):
        cc = self.max_errors
        s = 'Convergence criteria:\n'
        for name, val in [
            ('total energy change: {0:g} eV / electron',
             cc['energy'] * Ha / self.nvalence),
            ('integral of absolute density change: {0:g} electrons',
             cc['density'] / self.nvalence),
            ('integral of absolute eigenstate change: {0:g} eV^2',
             cc['eigenstates'] * Ha**2 / self.nvalence),
            ('change in atomic force: {0:g} eV / Ang',
             cc['force'] * Ha / Bohr),
            ('number of iterations: {0}', self.maxiter)]:
            if val < np.inf:
                s += '  Maximum {0}\n'.format(name.format(val))
        return s

    def write(self, writer):
        writer.write(converged=self.converged)

    def read(self, reader):
        self.converged = reader.scf.converged

    def reset(self):
        self.old_energies = []
        self.old_F_av = None
        self.converged = False

    def run(self, wfs, ham, dens, occ, log, callback):
        self.niter = 1
        while self.niter <= self.maxiter:
            wfs.eigensolver.iterate(ham, wfs)
            occ.calculate(wfs)

            energy = ham.get_energy(occ)
            self.old_energies.append(energy)
            errors = self.collect_errors(dens, ham, wfs)

            # Converged?
            for kind, error in errors.items():
                if error > self.max_errors[kind]:
                    self.converged = False
                    break
            else:
                self.converged = True

            callback(self.niter)
            self.log(log, self.niter, wfs, ham, dens, occ, errors)

            if self.converged and self.niter >= self.niter_fixdensity:
                break

            if self.niter > self.niter_fixdensity and not dens.fixed:
                dens.update(wfs)
                ham.update(dens)
            else:
                ham.npoisson = 0
            self.niter += 1

        # Don't fix the density in the next step:
        self.niter_fixdensity = 0

        if not self.converged:
            log(oops)
            raise KohnShamConvergenceError(
                'Did not converge!  See text output for help.')

    def collect_errors(self, dens, ham, wfs):
        """Check convergence of eigenstates, energy and density."""

        errors = {'eigenstates': wfs.eigensolver.error,
                  'density': dens.error,
                  'energy': np.inf}

        if dens.fixed:
            errors['density'] = 0.0

        if len(self.old_energies) >= 3:
            errors['energy'] = np.ptp(self.old_energies[-3:])

        # We only want to calculate the (expensive) forces if we have to:
        check_forces = (self.max_errors['force'] < np.inf and
                        all(error <= self.max_errors[kind]
                            for kind, error in errors.items()))

        errors['force'] = np.inf
        if check_forces:
            with wfs.timer('Forces'):
                F_av = calculate_forces(wfs, dens, ham)
            if self.old_F_av is not None:
                errors['force'] = ((F_av - self.old_F_av)**2).sum(1).max()**0.5
            self.old_F_av = F_av

        return errors

    def log(self, log, niter, wfs, ham, dens, occ, errors):
        """Output from each iteration."""

        nvalence = wfs.nvalence
        if nvalence > 0:
            eigerr = errors['eigenstates'] * Ha**2 / nvalence
        else:
            eigerr = 0.0

        T = time.localtime()

        if niter == 1:
            header = """\
                     log10-error:    total        iterations:
           time      wfs    density  energy       fermi  poisson"""
            if wfs.nspins == 2:
                header += '  magmom'
            if self.max_errors['force'] < np.inf:
                l1 = header.find('total')
                header = header[:l1] + '       ' + header[l1:]
                l2 = header.find('energy')
                header = header[:l2] + 'force  ' + header[l2:]
            log(header)

        if eigerr == 0.0:
            eigerr = ''
        else:
            eigerr = '%+.2f' % (ln(eigerr) / ln(10))

        denserr = errors['density']
        if denserr is None or denserr == 0 or nvalence == 0:
            denserr = ''
        else:
            denserr = '%+.2f' % (ln(denserr / nvalence) / ln(10))

        niterocc = occ.niter
        if niterocc == -1:
            niterocc = ''
        else:
            niterocc = '%d' % niterocc

        if ham.npoisson == 0:
            niterpoisson = ''
        else:
            niterpoisson = str(ham.npoisson)

        log('iter: %3d  %02d:%02d:%02d %6s %6s  ' %
            (niter,
             T[3], T[4], T[5],
             eigerr,
             denserr), end='')

        if self.max_errors['force'] < np.inf:
            if errors['force'] < np.inf:
                log('  %+.2f' %
                    (ln(errors['force'] * Ha / Bohr) / ln(10)), end='')
            else:
                log('       ', end='')

        log('%11.6f    %-5s  %-7s' %
            (Ha * ham.e_total_extrapolated,
             niterocc,
             niterpoisson), end='')

        if wfs.nspins == 2:
            log('  %+.4f' % occ.magmom, end='')
        elif not wfs.collinear:
            totmom_v, magmom_av = dens.estimate_magnetic_moments()
            log(' {:+.1f},{:+.1f},{:+.1f}'.format(*totmom_v), end='')

        log(flush=True)


oops = """
Did not converge!

Here are some tips:

1) Make sure the geometry and spin-state is physically sound.
2) Use less aggressive density mixing.
3) Solve the eigenvalue problem more accurately at each scf-step.
4) Use a smoother distribution function for the occupation numbers.
5) Try adding more empty states.
6) Use enough k-points.
7) Don't let your structure optimization algorithm take too large steps.
8) Solve the Poisson equation more accurately.
9) Better initial guess for the wave functions.

See details here:

    https://wiki.fysik.dtu.dk/gpaw/documentation/convergence.html

"""



import numpy as np
from ase import Atoms
from gpaw import GPAW, restart, setup_paths
from gpaw.lcao.tools import get_lcao_hamiltonian
from gpaw.mpi import world
from gpaw.atom.basis import BasisMaker
from gpaw.test import equal

if world.rank == 0:
    basis = BasisMaker('Li').generate(1, 1)
    basis.write_xml()
world.barrier()
if setup_paths[0] != '.':
    setup_paths.insert(0, '.')

if 1:
    a = 2.7
    bulk = Atoms('Li', pbc=True, cell=[a, a, a])
    calc = GPAW(gpts=(8, 8, 8), kpts=(4, 4, 4), mode='lcao', basis='szp')
    bulk.set_calculator(calc)
    e = bulk.get_potential_energy()
    niter = calc.get_number_of_iterations()
    calc.write('temp.gpw')

atoms, calc = restart('temp.gpw')
H_skMM, S_kMM = get_lcao_hamiltonian(calc)
eigs = calc.get_eigenvalues(kpt=2)

if world.rank == 0:
    eigs2 = sorted(np.linalg.eigvals(np.linalg.solve(S_kMM[2],
                                                     H_skMM[0, 2])).real)
    assert abs(sum(eigs - eigs2)) < 1e-8

    energy_tolerance = 0.0003
    niter_tolerance = 0
    equal(e, -1.82847, energy_tolerance)
    equal(niter, 5, niter_tolerance)



import numpy as np

from ase.build import molecule
from gpaw import GPAW
from gpaw.lcaotddft import LCAOTDDFT
from gpaw.poisson import PoissonSolver
from gpaw.lcaotddft.dipolemomentwriter import DipoleMomentWriter
from gpaw.mpi import world

from gpaw.test import equal

# Atoms
atoms = molecule('Na2')
atoms.center(vacuum=4.0)

# Ground-state calculation
calc = GPAW(nbands=2, h=0.4, setups=dict(Na='1'),
            basis='dzp', mode='lcao',
            poissonsolver=PoissonSolver('fd', eps=1e-16),
            convergence={'density': 1e-8},
            txt='gs.out')
atoms.set_calculator(calc)
energy = atoms.get_potential_energy()
calc.write('gs.gpw', mode='all')

# Time-propagation calculation with fxc
td_calc = LCAOTDDFT('gs.gpw', fxc='RPA', txt='td.out')
DipoleMomentWriter(td_calc, 'dm.dat')
td_calc.absorption_kick(np.ones(3) * 1e-5)
td_calc.propagate(20, 3)
world.barrier()

# Check dipole moment file
data = np.loadtxt('dm.dat')[:, 2:].ravel()
if 0:
    from gpaw.test import print_reference
    print_reference(data, 'ref', '%.12le')

ref = [-9.383700894739e-16,
       -9.338586948130e-16,
       2.131582675483e-14,
       8.679923327633e-15,
       7.529517689096e-15,
       2.074867751820e-14,
       1.960742702185e-05,
       1.960742702128e-05,
       1.804030746475e-05,
       3.761997205571e-05,
       3.761997205562e-05,
       3.596681520068e-05,
       5.257367158160e-05,
       5.257367158104e-05,
       5.366663490365e-05]

tol = 1e-9
equal(data, ref, tol)



import numpy as np
from ase.units import Ha, alpha, Bohr


s = np.array([[0.0]])
p = np.zeros((3, 3), complex)  # y, z, x
p[0, 1] = -1.0j
p[1, 0] = 1.0j
d = np.zeros((5, 5), complex)  # xy, yz, z^2, xz, x^2-y^2
d[0, 3] = -1.0j
d[3, 0] = 1.0j
d[1, 2] = -3**0.5 * 1.0j
d[2, 1] = 3**0.5 * 1.0j
d[1, 4] = -1.0j
d[4, 1] = 1.0j
Lx_lmm = [s, p, d]

p = np.zeros((3, 3), complex)  # y, z, x
p[1, 2] = -1.0j
p[2, 1] = 1.0j
d = np.zeros((5, 5), complex)  # xy, yz, z^2, xz, x^2-y^2
d[0, 1] = 1.0j
d[1, 0] = -1.0j
d[2, 3] = -3**0.5 * 1.0j
d[3, 2] = 3**0.5 * 1.0j
d[3, 4] = -1.0j
d[4, 3] = 1.0j
Ly_lmm = [s, p, d]

p = np.zeros((3, 3), complex)  # y, z, x
p[0, 2] = 1.0j
p[2, 0] = -1.0j
d = np.zeros((5, 5), complex)  # xy, yz, z^2, xz, x^2-y^2
d[0, 4] = 2.0j
d[4, 0] = -2.0j
d[1, 3] = 1.0j
d[3, 1] = -1.0j
Lz_lmm = [s, p, d]


def get_radial_potential(a, xc, D_sp):
    """Calculates dV/dr / r for the effective potential.
    Below, f_g denotes dV/dr = minus the radial force"""

    rgd = a.xc_correction.rgd
    r_g = rgd.r_g.copy()
    r_g[0] = 1.0e-12
    dr_g = rgd.dr_g

    B_pq = a.xc_correction.B_pqL[:, :, 0]
    n_qg = a.xc_correction.n_qg
    D_sq = np.dot(D_sp, B_pq)
    n_sg = np.dot(D_sq, n_qg) / (4 * np.pi)**0.5
    Ns = len(D_sp)
    if Ns == 4:
        Ns = 1
    n_sg[:Ns] += a.xc_correction.nc_g / Ns

    # Coulomb force from nucleus
    fc_g = a.Z / r_g**2

    # Hartree force
    rho_g = 4 * np.pi * r_g**2 * dr_g * np.sum(n_sg, axis=0)
    fh_g = -np.array([np.sum(rho_g[:ig]) for ig in range(len(r_g))]) / r_g**2

    f_g = fc_g + fh_g

    # xc force
    if xc.name != 'GLLBSC':
        v_sg = np.zeros_like(n_sg)
        xc.calculate_spherical(a.xc_correction.rgd, n_sg, v_sg)
        fxc_g = np.mean([a.xc_correction.rgd.derivative(v_g) for v_g in v_sg],
                        axis=0)
        f_g += fxc_g

    return f_g / r_g


def soc(a, xc, D_sp):
    v_g = get_radial_potential(a, xc, D_sp)
    Ng = len(v_g)
    phi_jg = a.data.phi_jg

    dVL_vii = np.zeros((3, a.ni, a.ni), complex)
    N1 = 0
    for j1, l1 in enumerate(a.l_j):
        Nm = 2 * l1 + 1
        N2 = 0
        for j2, l2 in enumerate(a.l_j):
            if l1 == l2:
                f_g = phi_jg[j1][:Ng] * v_g * phi_jg[j2][:Ng]
                c = a.xc_correction.rgd.integrate(f_g) / (4 * np.pi)
                dVL_vii[0, N1:N1 + Nm, N2:N2 + Nm] = c * Lx_lmm[l1]
                dVL_vii[1, N1:N1 + Nm, N2:N2 + Nm] = c * Ly_lmm[l1]
                dVL_vii[2, N1:N1 + Nm, N2:N2 + Nm] = c * Lz_lmm[l1]
            else:
                pass
            N2 += 2 * l2 + 1
        N1 += Nm
    return dVL_vii * alpha**2 / 4.0


def get_spinorbit_eigenvalues(calc, bands=None, gw_kn=None, return_spin=False,
                              return_wfs=False, scale=1.0,
                              theta=0.0, phi=0.0):
    """
    Parameters:
        calc: Calculator
            GPAW calculator
        bands: list of ints
            list of band indices for which to calculate soc.
        gw_kn: (ns, nk, nb) or (nk, nb)-shape array
            use eigenvalues in gw_kn instead of from calc.get_eigenvalues
        return_spin: bool
            should the spin projection be calculated and returned.
        return_wfs: bool
            flag for returning wave functions
        scale: float
            scale the spinorbit coupling by this amount
        theta: float
            angle in radians
        phi: float
            angle in radians
    Returns:
        out: e_mk or (e_mk, s_kvm) or (e_mk, wfs_knm) or (e_mk, s_kvm, wfs_knm)
   """

    if bands is None:
        bands = np.arange(calc.get_number_of_bands())

    Na = len(calc.atoms)
    Nk = len(calc.get_ibz_k_points())
    Ns = calc.wfs.nspins
    Nn = len(bands)
    noncollinear = not calc.density.collinear
    if noncollinear:
        Nn //= 2

    if gw_kn is not None:
        gw_skn = gw_kn.copy()
        if gw_skn.ndim == 2:  # it is gw_kn
            gw_skn = gw_skn[np.newaxis]
        assert Ns == gw_skn.shape[0]
        assert Nk == gw_skn.shape[1]
        assert Nn == gw_skn.shape[2]

    if Ns == 1:
        if gw_kn is None:
            e_kn = [calc.get_eigenvalues(kpt=k)[bands] for k in range(Nk)]
        else:
            e_kn = gw_skn[0]
        e_skn = np.array([e_kn, e_kn])
    else:
        if gw_kn is None:
            e_skn = np.array([[calc.get_eigenvalues(kpt=k, spin=s)[bands]
                               for k in range(Nk)] for s in range(2)])
        else:
            e_skn = gw_skn.copy()

    # <phi_i|dV_adr / r * L_v|phi_j>
    dVL_avii = []
    for ai in range(Na):
        a = calc.wfs.setups[ai]
        dVL_avii.append(soc(a, calc.hamiltonian.xc, calc.density.D_asp[ai]))

    e_km = []
    if return_wfs:
        v_knm = []
    if return_spin:
        v_knm = []
        s_kvm = []

    # Hamiltonian with SO in KS basis
    # The even indices in H_mm are spin up along \hat n defined by \theta, phi
    # Basis change matrix for constructing Pauli matrices in \theta,\phi basis:
    #     \sigma_i^n = C^\dag\sigma_i C
    C_ss = np.array([[np.cos(theta / 2) * np.exp(-1.0j * phi / 2),
                      -np.sin(theta / 2) * np.exp(-1.0j * phi / 2)],
                     [np.sin(theta / 2) * np.exp(1.0j * phi / 2),
                      np.cos(theta / 2) * np.exp(1.0j * phi / 2)]])
    sx_ss = np.array([[0, 1], [1, 0]], complex)
    sy_ss = np.array([[0, -1.0j], [1.0j, 0]], complex)
    sz_ss = np.array([[1, 0], [0, -1]], complex)
    sx_ss = C_ss.T.conj().dot(sx_ss).dot(C_ss)
    sy_ss = C_ss.T.conj().dot(sy_ss).dot(C_ss)
    sz_ss = C_ss.T.conj().dot(sz_ss).dot(C_ss)

    for k in range(Nk):
        # Evaluate H in a basis of S_z eigenstates
        H_mm = np.zeros((2 * Nn, 2 * Nn), complex)
        if not noncollinear:
            i1 = np.arange(0, 2 * Nn, 2)
            i2 = np.arange(1, 2 * Nn, 2)
            H_mm[i1, i1] = e_skn[0, k]
            H_mm[i2, i2] = e_skn[1, k]
        else:
            i0 = np.arange(0, 2 * Nn)
            H_mm[i0, i0] = e_skn[0, k]
        for ai in range(Na):
            if not noncollinear:
                Pt_sni = [calc.wfs.kpt_u[k + s * Nk].P_ani[ai][bands]
                          for s in range(Ns)]
            else:
                Pt_sni = np.swapaxes(calc.wfs.kpt_u[k].P_ani[ai], 0, 1)
            Ni = len(Pt_sni[0][0])
            P_sni = np.zeros((2, 2 * Nn, Ni), complex)
            dVL_vii = dVL_avii[ai] * scale * Ha
            if Ns == 1:
                if not noncollinear:
                    P_sni[0, ::2] = Pt_sni[0]
                    P_sni[1, 1::2] = Pt_sni[0]
                else:
                    P_sni = Pt_sni
            else:
                P_sni[0, ::2] = Pt_sni[0]
                P_sni[1, 1::2] = Pt_sni[1]
            H_ssii = np.zeros((2, 2, Ni, Ni), complex)
            H_ssii[0, 0] = dVL_vii[2]
            H_ssii[0, 1] = dVL_vii[0] - 1.0j * dVL_vii[1]
            H_ssii[1, 0] = dVL_vii[0] + 1.0j * dVL_vii[1]
            H_ssii[1, 1] = -dVL_vii[2]

            # Tranform to theta, phi basis
            H_ssii = np.tensordot(C_ss, H_ssii, ([0, 1]))
            H_ssii = np.tensordot(C_ss.T.conj(), H_ssii, ([1, 1]))
            for s1 in range(2):
                for s2 in range(2):
                    H_ii = H_ssii[s1, s2]
                    P1_mi = P_sni[s1]
                    P2_mi = P_sni[s2]
                    H_mm += np.dot(np.dot(P1_mi.conj(), H_ii), P2_mi.T)

        e_m, v_snm = np.linalg.eigh(H_mm)
        e_km.append(e_m)
        if return_wfs or return_spin:
            v_knm.append(v_snm)
        if return_spin:
            sx_m = []
            sy_m = []
            sz_m = []
            for m in range(2 * Nn):
                v_sn = np.array([v_snm[::2, m], v_snm[1::2, m]])
                sx_m.append(np.trace(v_sn.T.conj().dot(sx_ss).dot(v_sn)))
                sy_m.append(np.trace(v_sn.T.conj().dot(sy_ss).dot(v_sn)))
                sz_m.append(np.trace(v_sn.T.conj().dot(sz_ss).dot(v_sn)))
            s_kvm.append([sx_m, sy_m, sz_m])

    if return_spin:
        if return_wfs:
            return np.array(e_km).T, np.array(s_kvm).real, v_knm
        else:
            return np.array(e_km).T, np.array(s_kvm).real
    else:
        if return_wfs:
            return np.array(e_km).T, v_knm
        else:
            return np.array(e_km).T


def set_calculator(calc, e_km, v_knm=None, width=None):
    from gpaw.occupations import FermiDirac
    from ase.units import Hartree

    noncollinear = not calc.density.collinear

    if width is None:
        width = calc.occupations.width * Hartree
    if not noncollinear:
        calc.wfs.bd.nbands *= 2
    # calc.wfs.nspins = 1
    for kpt in calc.wfs.kpt_u:
        kpt.eps_n = e_km[kpt.k] / Hartree
        kpt.f_n = np.zeros_like(kpt.eps_n)
        if not noncollinear:
            kpt.weight /= 2
    ef = calc.occupations.fermilevel
    calc.occupations = FermiDirac(width)
    calc.occupations.nvalence = calc.wfs.setups.nvalence - calc.density.charge
    calc.occupations.fermilevel = ef
    calc.occupations.calculate_occupation_numbers(calc.wfs)
    for kpt in calc.wfs.kpt_u:
        kpt.f_n *= 2
        kpt.weight *= 2


def get_anisotropy(calc, theta=0.0, phi=0.0, nbands=None, width=None):
    """Calculates the sum of occupied spinorbit eigenvalues. Returns the result
    relative to the sum of eigenvalues without spinorbit coupling"""

    Ns = calc.wfs.nspins
    noncollinear = not calc.density.collinear

    e_skn = np.array([[calc.get_eigenvalues(kpt=k, spin=s)
                       for k in range(len(calc.get_ibz_k_points()))]
                      for s in range(Ns)])
    e_kn = np.reshape(np.swapaxes(e_skn, 0, 1), (len(e_skn[0]),
                                                 Ns * len(e_skn[0, 0])))
    e_kn = np.sort(e_kn, 1)
    if nbands is None:
        nbands = len(e_skn[0, 0])
    f_skn = np.array([[calc.get_occupation_numbers(kpt=k, spin=s)
                       for k in range(len(calc.get_ibz_k_points()))]
                      for s in range(Ns)])
    f_kn = np.reshape(np.swapaxes(f_skn, 0, 1), (len(f_skn[0]),
                                                 Ns * len(f_skn[0, 0])))
    f_kn = np.sort(f_kn, 1)[:, ::-1]
    if noncollinear:
        f_kn *= 2

    E = np.sum(e_kn * f_kn)

    from gpaw.occupations import occupation_numbers
    e_mk = get_spinorbit_eigenvalues(calc, theta=theta, phi=phi,
                                     bands=range(nbands))
    if width is None:
        width = calc.occupations.width * Ha
    if width == 0.0:
        width = 1.e-6
    weight_k = calc.get_k_point_weights() / 2
    ne = calc.wfs.setups.nvalence - calc.density.charge
    f_km = occupation_numbers({'name': 'fermi-dirac', 'width': width},
                              np.array([e_mk.T]),
                              weight_k=weight_k,
                              nelectrons=ne)[0][0]
    E_so = np.sum(e_mk.T * f_km)
    return E_so - E


def get_spinorbit_projections(calc, ik, v_nm):
    # For spinors the number of projectors and bands are doubled
    Na = len(calc.atoms)
    Nk = len(calc.get_ibz_k_points())
    Ns = calc.wfs.nspins

    v0_mn = v_nm[::2].T
    v1_mn = v_nm[1::2].T
        
    P_ani = {}
    for ia in range(Na):
        P0_ni = calc.wfs.kpt_u[ik].P_ani[ia]
        P1_ni = calc.wfs.kpt_u[(Ns - 1) * Nk + ik].P_ani[ia]

        P0_mi = np.dot(v0_mn, P0_ni)
        P1_mi = np.dot(v1_mn, P1_ni)
        P_mi = np.zeros((len(P0_mi), 2 * len(P0_mi[0])), complex)
        P_mi[:, ::2] = P0_mi
        P_mi[:, 1::2] = P1_mi
        P_ani[ia] = P_mi

    return P_ani

    
def get_spinorbit_wavefunctions(calc, ik, v_nm):
    # For spinors the number of bands is doubled and a spin dimension is added
    Ns = calc.wfs.nspins
    Nn = calc.wfs.bd.nbands

    v0_mn = v_nm[::2].T
    v1_mn = v_nm[1::2].T
    
    u0_nG = np.array([calc.wfs.get_wave_function_array(n, ik, 0)
                      for n in range(Nn)])
    u1_nG = np.array([calc.wfs.get_wave_function_array(n, ik, (Ns - 1))
                      for n in range(Nn)])
    u0_mG = np.swapaxes(np.dot(v0_mn, np.swapaxes(u0_nG, 0, 2)), 1, 2)
    u1_mG = np.swapaxes(np.dot(v1_mn, np.swapaxes(u1_nG, 0, 2)), 1, 2)
    u_mG = np.zeros((len(u0_mG),
                     2,
                     len(u0_mG[0]),
                     len(u0_mG[0, 0]),
                     len(u0_mG[0, 0, 0])), complex)
    u_mG[:, 0] = u0_mG
    u_mG[:, 1] = u1_mG

    return u_mG

    
def get_magnetic_moments(calc, theta=0.0, phi=0.0, nbands=None, width=None):
    """Calculates the magnetic moments inside all PAW spheres"""

    from gpaw.utilities import unpack

    if nbands is None:
        nbands = calc.get_number_of_bands()
    Nk = len(calc.get_ibz_k_points())

    C_ss = np.array([[np.cos(theta / 2) * np.exp(-1.0j * phi / 2),
                      -np.sin(theta / 2) * np.exp(-1.0j * phi / 2)],
                     [np.sin(theta / 2) * np.exp(1.0j * phi / 2),
                      np.cos(theta / 2) * np.exp(1.0j * phi / 2)]])
    sx_ss = np.array([[0, 1], [1, 0]], complex)
    sy_ss = np.array([[0, -1.0j], [1.0j, 0]], complex)
    sz_ss = np.array([[1, 0], [0, -1]], complex)
    sx_ss = C_ss.T.conj().dot(sx_ss).dot(C_ss)
    sy_ss = C_ss.T.conj().dot(sy_ss).dot(C_ss)
    sz_ss = C_ss.T.conj().dot(sz_ss).dot(C_ss)

    e_mk, v_knm = get_spinorbit_eigenvalues(calc,
                                            theta=theta,
                                            phi=phi,
                                            return_wfs=True,
                                            bands=range(nbands))

    from gpaw.occupations import occupation_numbers
    if width is None:
        width = calc.occupations.width * Ha
    if width == 0.0:
        width = 1.e-6
    weight_k = calc.get_k_point_weights() / 2
    ne = calc.wfs.setups.nvalence - calc.density.charge
    f_km = occupation_numbers({'name': 'fermi-dirac', 'width': width},
                              np.array([e_mk.T]),
                              weight_k=weight_k,
                              nelectrons=ne)[0][0]

    m_v = np.zeros(3, complex)
    for ik in range(Nk):
        ut0_nG = np.array([calc.wfs.get_wave_function_array(n, ik, 0)
                           for n in range(nbands)])
        ut1_nG = np.array([calc.wfs.get_wave_function_array(n, ik, 1)
                           for n in range(nbands)])
        mocc = np.where(f_km[ik] * Nk - 1.0e-6 < 0.0)[0][0]
        for m in range(mocc + 1):
            f = f_km[ik, m]
            ut0_G = np.dot(v_knm[ik][::2, m], np.swapaxes(ut0_nG, 0, 2))
            ut1_G = np.dot(v_knm[ik][1::2, m], np.swapaxes(ut1_nG, 0, 2))
            ut_sG = np.array([ut0_G, ut1_G])

            mx_G = np.zeros(np.shape(ut0_G), complex)
            my_G = np.zeros(np.shape(ut0_G), complex)
            mz_G = np.zeros(np.shape(ut0_G), complex)
            for s1 in range(2):
                for s2 in range(2):
                    mx_G += ut_sG[s1].conj() * sx_ss[s1, s2] * ut_sG[s2]
                    my_G += ut_sG[s1].conj() * sy_ss[s1, s2] * ut_sG[s2]
                    mz_G += ut_sG[s1].conj() * sz_ss[s1, s2] * ut_sG[s2]
            m_v += calc.wfs.gd.integrate(np.array([mx_G, my_G, mz_G])) * f

    m_av = []
    for a in range(len(calc.atoms)):
        N0_p = calc.density.setups[a].N0_p.copy()
        N0_ij = unpack(N0_p)
        Dx_ij = np.zeros_like(N0_ij, complex)
        Dy_ij = np.zeros_like(N0_ij, complex)
        Dz_ij = np.zeros_like(N0_ij, complex)
        Delta_p = calc.density.setups[a].Delta_pL[:, 0].copy()
        Delta_ij = unpack(Delta_p)
        for ik in range(Nk):
            P_ami = get_spinorbit_projections(calc, ik, v_knm[ik])
            P_smi = np.array([P_ami[a][:, ::2], P_ami[a][:, 1::2]])
            P_smi = np.dot(C_ss, np.swapaxes(P_smi, 0, 1))

            P0_mi = P_smi[0]
            P1_mi = P_smi[1]
            f_mm = np.diag(f_km[ik])

            Dx_ij += P0_mi.conj().T.dot(f_mm).dot(P1_mi)
            Dx_ij += P1_mi.conj().T.dot(f_mm).dot(P0_mi)
            Dy_ij -= 1.0j * P0_mi.conj().T.dot(f_mm).dot(P1_mi)
            Dy_ij += 1.0j * P1_mi.conj().T.dot(f_mm).dot(P0_mi)
            Dz_ij += P0_mi.conj().T.dot(f_mm).dot(P0_mi)
            Dz_ij -= P1_mi.conj().T.dot(f_mm).dot(P1_mi)

        mx = np.sum(N0_ij * Dx_ij).real
        my = np.sum(N0_ij * Dy_ij).real
        mz = np.sum(N0_ij * Dz_ij).real

        m_av.append([mx, my, mz])
        m_v[0] += np.sum(Delta_ij * Dx_ij) * (4 * np.pi)**0.5
        m_v[1] += np.sum(Delta_ij * Dy_ij) * (4 * np.pi)**0.5
        m_v[2] += np.sum(Delta_ij * Dz_ij) * (4 * np.pi)**0.5

    return m_v.real, m_av


def get_parity_eigenvalues(calc, ik=0, spin_orbit=False, bands=None, Nv=None,
                           inversion_center=[0, 0, 0], deg_tol=1.0e-6,
                           tol=1.0e-6):
    """Calculates parity eigenvalues at time-reversal invariant k-points.
    Only works in plane wave mode.
    """

    kpt_c = calc.get_ibz_k_points()[ik]
    if Nv is None:
        Nv = int(calc.get_number_of_electrons() / 2)

    if bands is None:
        bands = range(calc.get_number_of_bands())

    # Find degenerate subspaces
    eig_n = calc.get_eigenvalues(kpt=ik)[bands]
    e_in = []
    used_n = []
    for n1, e1 in enumerate(eig_n):
        if n1 not in used_n:
            n_n = []
            for n2, e2 in enumerate(eig_n):
                if np.abs(e1 - e2) < deg_tol:
                    n_n.append(n2)
                    used_n.append(n2)
            e_in.append(n_n)

    print()
    print(' Inversion center at: %s' % inversion_center)
    print(' Calculating inversion eigenvalues at k = %s' % kpt_c)
    print()

    center_v = np.array(inversion_center) / Bohr
    G_Gv = calc.wfs.pd.get_reciprocal_vectors(q=ik, add_q=True)

    psit_nG = np.array([calc.wfs.kpt_u[ik].psit_nG[n]
                        for n in bands])
    if spin_orbit:
        e_nk, v_knm = get_spinorbit_eigenvalues(calc, return_wfs=True,
                                                bands=bands)
        psit0_mG = np.dot(v_knm[ik][::2].T, psit_nG)
        psit1_mG = np.dot(v_knm[ik][1::2].T, psit_nG)
    for n in range(len(bands)):
        psit_nG[n] /= (np.sum(np.abs(psit_nG[n])**2))**0.5
    if spin_orbit:
        for n in range(2 * len(bands)):
            A = np.sum(np.abs(psit0_mG[n])**2) + np.sum(np.abs(psit1_mG[n])**2)
            psit0_mG[n] /= A**0.5
            psit1_mG[n] /= A**0.5

    P_GG = np.ones((len(G_Gv), len(G_Gv)), float)
    for iG, G_v in enumerate(G_Gv):
        P_GG[iG] -= ((G_Gv[:] + G_v).round(6)).any(axis=1)
    assert (P_GG == P_GG.T).all()

    phase_G = np.exp(-2.0j * np.dot(G_Gv, center_v))

    p_n = []
    print('n   P_n')
    for n_n in e_in:
        if spin_orbit:
            # The dimension of parity matrix is doubled with spinorbit
            m_m = [2 * n_n[0] + i for i in range(2 * len(n_n))]
            Ppsit0_mG = np.dot(P_GG, psit0_mG[m_m].T).T
            Ppsit0_mG[:] *= phase_G
            Ppsit1_mG = np.dot(P_GG, psit1_mG[m_m].T).T
            Ppsit1_mG[:] *= phase_G
            P_nn = np.dot(psit0_mG[m_m].conj(), np.array(Ppsit0_mG).T)
            P_nn += np.dot(psit1_mG[m_m].conj(), np.array(Ppsit1_mG).T)
        else:
            Ppsit_nG = np.dot(P_GG, psit_nG[n_n].T).T
            Ppsit_nG[:] *= phase_G
            P_nn = np.dot(psit_nG[n_n].conj(), np.array(Ppsit_nG).T)
        P_eig = np.linalg.eigh(P_nn)[0]
        if np.allclose(np.abs(P_eig), 1, tol):
            P_n = np.sign(P_eig).astype(int).tolist()
            if spin_orbit:
                # Only include one of the degenerate pair of eigenvalues
                Pm = np.sign(P_eig).tolist().count(-1)
                Pp = np.sign(P_eig).tolist().count(1)
                P_n = Pm // 2 * [-1] + Pp // 2 * [1]
            print('%s: %s' % (str(n_n)[1:-1], str(P_n)[1:-1]))
            p_n += P_n
        else:
            print('  %s are not parity eigenstates' % n_n)
            print('     P_n: %s' % P_eig)
            print('     e_n: %s' % eig_n[n_n])
            p_n += [0 for n in n_n]

    return np.ravel(p_n)



import numpy as np
from ase import Atoms
from gpaw import GPAW, PW, MixerDif


def create_tasks():
    from myqueue.task import task
    return [task('cr.agts.py', cores=8)]


def angle(a, b):
    x = np.dot(a, b) / (np.dot(a, a) * np.dot(b, b))**0.5
    return np.arccos(x) * 180 / np.pi


def check(m_av):
    for a in range(3):
        theta = angle(m_av[a], m_av[a - 1])
        print(theta)
        assert abs(theta - 120) < 0.5, theta


if __name__ == '__main__':
    d = 2.66
    atoms = Atoms('Cr3', positions=[(0, 0, 0), (d, 0, 0), (2 * d, 0, 0)],
                  cell=[[d * 3 / 2, -d * np.sqrt(3) / 2, 0],
                        [d * 3 / 2, d * np.sqrt(3) / 2, 0],
                        [0, 0, 6]],
                  pbc=True)

    magmoms = [[3, 3, 0], [3, -1, 0], [-4, 0, 1.0]]

    calc = GPAW(mode=PW(400),
                symmetry='off',
                mixer=MixerDif(),
                experimental={'magmoms': magmoms},
                kpts=(4, 4, 1))
    atoms.calc = calc
    atoms.get_potential_energy()

    _, m_av = calc.density.estimate_magnetic_moments()
    check(m_av)

    calc.write('Cr3.gpw')
    calc = GPAW('Cr3.gpw', txt=None)

    _, m_av = calc.density.estimate_magnetic_moments()
    check(m_av)



from math import pi
import numpy as np
from gpaw.xc.libxc import LibXC

x0 = LibXC('LDA_X')

def f0(xc, rs, s):
    n = 3 / (4 * pi * rs**3)
    third = 1.0 / 3.0
    kF = (3 * pi**2 * n)**third
    a2 = (2 * kF * n * s)**2
    e = np.zeros(1)
    xc.calculate(e,
                 np.array([[n]]), np.zeros((1, 1)),
                 np.array([[a2]]), np.zeros((1, 1)))
    exc = n * e[0]
    x0.calculate(e, np.array([[n]]), np.zeros((1, 1)))
    ex0 = n * e[0]
    return exc / ex0

def f1(xc, rs, s):
    n = 3 / (4 * pi * rs**3)
    # na = 2 * n
    third = 1.0 / 3.0
    kF = (3 * pi**2 * n)**third
    a2 = (2 * kF * n * s)**2
    e = np.zeros(1)
    xc.calculate(e,
                 np.array([[n], [0]]), np.zeros((2, 1)),
                 np.array([[a2], [0], [0]]), np.zeros((3, 1)))
    exc = n * e[0]
    x0.calculate(e, np.array([[n]]), np.zeros((1, 1)))
    ex0 = n * e[0]
    return exc / ex0

pbe = LibXC('PBE')
pw91 = LibXC('PW91')
assert abs(f0(pbe, 2, 3) - 1.58) < 0.01
assert abs(f1(pbe, 2, 3) - 1.88) < 0.01
assert abs(f0(pw91, 2, 3) - 1.60) < 0.01
assert abs(f1(pw91, 2, 3) - 1.90) < 0.01
